"""
The GUI model editor.

Tools:   for the editor menu bar
Objects: that can be manipulated in the canvas
Window:  window and canvas 

Originally written by Alan Lindsay.

$Id$
"""
__version__='$Revision: 8989 $'

from inspect import getdoc
import math

from Tkinter import Button, Label, Frame, TOP, LEFT, RIGHT, BOTTOM, E, LAST, FIRST, OptionMenu, StringVar,Canvas,X,GROOVE,RAISED,Checkbutton,Menu,Scrollbar, YES,Y,END,BOTH
from tkFileDialog import asksaveasfilename


import param
from param import parameterized,normalize_path
import paramtk as tk
from paramtk.tilewrapper import Combobox

import topo
from topo.command.analysis import update_activity
from topo.misc.util import shortclassname
from topo.base.simulation import EventProcessor

# Make sure at least some sheets or projections are available to
# choose from; rest will be available if some other module imports *
# from sheet and projection
from topo.base.sheet import Sheet
from topo.base.projection import Projection
from topo.base.cf import CFProjection


###########################################################################
## WINDOW
###########################################################################

# These can be customized, e.g. in .topographicarc; they should probably be parameters
# somewhere.
canvas_width = 1200
scaling_factor = topo.sim.item_scale
enlarging_factor = 1.25

canvas_region = (0, 0, canvas_width, canvas_width)
"""Size of the canvas, as a bounding box (xl yl xh yh)."""


class EditorCanvas(Canvas):

    """ 
    EditorCanvas extends the Tk Canvas class.
    There are 3 modes that determine the effect of mouse events in the Canvas
    A Canvas can accept new objects, move objects and make connections 
    between them. The intended use of this class is as the main
    canvas in a Topographica model-editing GUI. 
    """

    def __init__(self, root = None, width = 600, height = 600):
        Canvas.__init__(self, root, width = width, height = height,bg='white')
        # bg = "white", bd = 2, relief = SUNKEN)
        self.panel = Frame(root)
        self.panel.pack(side = TOP, fill = X)
        # Top bar of the canvas, allowing changes in size, display and to force refresh.
        Button(self.panel,text="Refresh", command=self.refresh).pack(side=LEFT)
        Button(self.panel,text="Reduce", command=self.reduce_scale).pack(side=LEFT)
        Button(self.panel,text="Enlarge", command=self.enlarge_scale).pack(side=LEFT)        
        self.auto_refresh = False
        self.console = topo.guimain
        self.auto_refresh_checkbutton = Checkbutton(self.panel,text="Auto-refresh",
                                                    command=self.toggle_auto_refresh)
        self.auto_refresh_checkbutton.pack(side=LEFT)

        self.normalize_checkbutton = Checkbutton(self.panel, text="Normalize",
                                                    command=self.toggle_normalize)
        self.normalize_checkbutton.pack(side=LEFT)
        if EditorSheet.normalize == True:
            self.normalize_checkbutton.select()

        self.node_labels_checkbutton = Checkbutton(self.panel, text="Node labels",
                                                    command=self.toggle_node_labels)
        self.node_labels_checkbutton.pack(side=LEFT)
        if EditorNode.show_label == True:
            self.node_labels_checkbutton.select()

        self.connection_labels_checkbutton = Checkbutton(self.panel, text="Connection labels",
                                                    command=self.toggle_connection_labels)
        self.connection_labels_checkbutton.pack(side=LEFT)
        if EditorConnection.show_label == True:
            self.connection_labels_checkbutton.select()

        # retain the current focus in the canvas
        self.scaling_factor = topo.sim.item_scale#scaling_factor
        self.current_object = None
        self.current_connection = None
        self.focus = None
        # list holding references to all the objects in the canvas      
        self.object_list = []
        # set the initial mode.
        self.display_mode = 'video'
        self.mode = "ARROW"
        self.MAX_VIEWS = 5
        # get the topo simulation
        self.simulation = topo.sim

        # create the menu widget used as a popup on objects and connections
        self.option_add("*Menu.tearOff", "0") 
        self.item_menu = Menu(self)
        self.view = Menu(self.item_menu)
        # add property, toggle activity drawn on sheets, object draw ordering and delete entries to the menu.
        self.item_menu.insert_command(END, label = 'Properties', 
            command = lambda: self.show_properties(self.focus))
        self.item_menu.add_cascade(label = 'Change View', menu = self.view, underline = 0)
        self.item_menu.insert_command(END, label = 'Move Forward', 
            command = lambda: self.move_forward(self.focus))
        self.item_menu.insert_command(END, label = 'Move to Front', 
            command = lambda: self.move_to_front(self.focus))
        self.item_menu.insert_command(END, label = 'Move to Back', 
            command = lambda: self.move_to_back(self.focus))
        self.item_menu.insert_command(END, label = 'Delete', 
            command = lambda: self.delete_focus(self.focus))
        # the indexes of the menu items that are for objects only
        self.object_indices = [2,3,4]

        self.canvas_menu = Menu(self)
        self.sheet_options = Menu(self.canvas_menu)
        mode_options = Menu(self.canvas_menu)
        self.canvas_menu.add_command(label = 'Export as PostScript image', command = self.save_snapshot)
        self.canvas_menu.add_cascade(label = 'Select Mode', menu = mode_options)
        mode_options.add_command(label = 'Video', command = lambda: self.set_display_mode('video'))
        mode_options.add_command(label = 'Normal', command = lambda: self.set_display_mode('normal'))
        mode_options.add_command(label = 'Printing', command = lambda: 
            self.set_display_mode('printing'))
        self.canvas_menu.add_cascade(label = 'Sheet options', menu = self.sheet_options, underline = 0)
        self.sheet_options.add_command(label = 'Toggle Density Grid', command = 
            self.toggle_object_density)
        self.sheet_options.add_command(label = 'Toggle Activity', command = self.toggle_object_activity)

        # bind key_press events in canvas.
        self.bind('<KeyPress>', self.key_press)
        # bind the possible left button events to the canvas.
        self.bind('<Button-1>', self.left_click)
        self.bind('<B1-Motion>', self.left_click_drag)
        self.bind('<Double-1>', self.left_double_click)
        self.bind('<ButtonRelease-1>', self.left_release)
        # bind the possible right button events to the canvas.
        self.bind('<<right-click>>', self.right_click)
        # because right-click opens menu, a release event can only be flagged by the menu.
        self.item_menu.bind('<<right-click-release>>', self.right_release)   

        # add scroll bar; horizontal and vertical
        self.config(scrollregion = canvas_region)
        vertical_scrollbar = Scrollbar(root)
        horizontal_scrollbar = Scrollbar(root, orient = 'horizontal')
        vertical_scrollbar.config(command = self.yview)
        horizontal_scrollbar.config(command = self.xview)
        self.config(yscrollcommand=vertical_scrollbar.set)
        self.config(xscrollcommand=horizontal_scrollbar.set)
        vertical_scrollbar.pack(side = RIGHT, fill = Y)
        horizontal_scrollbar.pack(side = BOTTOM, fill = X)
    
    def key_press(self, event):
        "What happens when a key is pressed."
        self.change_mode(event.char)


    #   Left mouse button event handlers
        
    def left_click(self, event):
        "What is to happen if the left button is pressed."
        
        x,y = self.canvasx(event.x), self.canvasy(event.y)
        {"ARROW" : self.init_move,           # case Arrow mode
         "MAKE" : self.none,                 # case Make mode
         "CONNECTION" : self.init_connection # case Connection mode.
        }[self.mode](x,y)                    # select function depending on mode
    
    def left_click_drag(self, event):
        "What is to happen if the mouse is dragged while the left button is pressed."
        
        x,y = self.canvasx(event.x), self.canvasy(event.y)
        {"ARROW" : self.update_move,              # case Arrow mode
         "MAKE" : self.none,                   # case Make mode
         "CONNECTION" : self.update_connection # case Connection mode.
        }[self.mode](x,y)                      # select function depending on mode
        
    def left_release(self, event):
        "What is to happen when the left mouse button is released."
        
        x,y = self.canvasx(event.x), self.canvasy(event.y)
        {"ARROW" : self.end_move,            # case Arrow mode
         "MAKE" : self.create_object,        # case Make mode
         "CONNECTION" : self.end_connection  # case Connection mode.
        }[self.mode](x,y)                    # select function depending on mode

    def left_double_click(self, event):
        """
        What is to happen if the left button is double clicked.
        The same for all modes - show the properties for the clicked item.
        Gets object or connection at this point and gives it the focus.
        """
        focus = self.get_xy(event.x, event.y) 
        if (focus != None):
            focus.set_focus(True)
        # show the object or connection's properties.
        self.show_properties(focus)         

    
    #   Right mouse button event handlers

    def right_click(self, event):
        "What is to happen if the right button is pressed."
        self.show_hang_list(event)

    def right_release(self, event):
        "What is to happen when the right mouse button is released (bound to the menu)."
        if (self.focus != None) : # remove focus.
            self.focus.set_focus(False)


    #   Mode Methods
    
    def change_mode(self, char):
        "Changes the mode of the canvas, i.e., what mouse events will do."
        
        if not char in ('c', 'm', 'a') : return
        # remove the focus from the previous toolbar item
        {"ARROW" : self.arrow_tool.set_focus,     # arrow toolbar item
         "MAKE" : self.object_tool.set_focus,        # object toolbar item
         "CONNECTION" : self.connection_tool.set_focus # connection toolbar item
        }[self.mode](False)                  # select function depending on mode

        # determine the new mode and corresponding toolbar item.
        if (char == 'c'):
            mode = "CONNECTION"
            bar = self.connection_tool
        elif (char == 'm'):
            mode = "MAKE" 
            bar = self.object_tool
        elif (char == 'a'):
            mode = "ARROW"
            bar = self.arrow_tool
        # set the focus of the toolbar item of the new mode and retain the note the new mode.
        bar.set_focus(True)
        self.mode = mode


    #   Panel methods
    
    def refresh(self):
        for obj in self.object_list:
            obj.set_focus(True)
            obj.set_focus(False)
        for obj in self.object_list:
            connection_list = obj.from_connections[:]
            connection_list.reverse()
            for con in connection_list:
                con.move()

    def enlarge_scale(self):
        self.scaling_factor *= enlarging_factor
        self.refresh()
 
    def reduce_scale(self):
        self.scaling_factor /= enlarging_factor
        self.refresh()

    def toggle_auto_refresh(self):
        self.auto_refresh = not self.auto_refresh
        if self.auto_refresh:
            self.console.auto_refresh_panels.append(self)
        else:
            self.console.auto_refresh_panels.remove(self)

    def toggle_normalize(self):
        EditorSheet.normalize = not EditorSheet.normalize
        self.refresh()

    def toggle_node_labels(self):
        EditorNode.show_label = not EditorNode.show_label
        self.refresh()

    def toggle_connection_labels(self):
        EditorConnection.show_label = not EditorConnection.show_label
        self.refresh()



    #   Object moving methods
    #
    # If an object is left clicked in the canvas, these methods allow
    # it to be repositioned in the canvas.

    def init_move(self, x, y):
        "Determine if click was on an object."
        
        self.current_object = self.get_object_xy(x, y)
        if (self.current_object != None) : 
            # if it was, give it the focus
            self.current_object.set_focus(True)

    def update_move(self, x, y):
        "If dragging an object, refresh its position"
        
        if (self.current_object != None):
            self.current_object.move(x, y)

    def end_move(self, x, y):
        "If dropping an object, remove focus and refresh."
        
        if (self.current_object != None):
            self.current_object.set_focus(False)
            self.current_object.move(x, y)
        # redraw all the objects in the canavas and dereference
        self.redraw_objects()
        self.current_object = None

    #   Connection methods
    #
    # these methods allow a connection to be made between two objects in the canvas

    def init_connection(self, x, y):
        "Determine if click was on an object, and retain if so."
        
        current_object = self.get_object_xy(x, y)
        if (current_object == None) : # if not change to ARROW mode
            self.change_mode('a')
        else :  # if on an object, create a connection and give it the focus
            self.current_connection = self.connection_tool.new_cover(current_object)
            self.current_connection.set_focus(True)

    def update_connection(self, x, y):
        "Update connection's position."
        self.current_connection.update_position((x, y))
        
    def end_connection(self, x, y):
        "Determine if the connection has been dropped on an object."
        
        obj = self.get_object_xy(x, y)
        if (obj != None) : # if an object, connect the objects and remove focus
            if (self.current_connection != None):
                connected = self.connection_tool.create_connection(self.current_connection, obj)
                if connected:
                    self.current_connection.set_focus(False)
        else : # if not an object, remove the connection
            connected=False
            if (self.current_connection != None):
                self.current_connection.remove()
        if connected:
            self.redraw_objects()
        # dereference
        self.current_connection = None

    def get_connection_xy(self, x, y):
        "Return connection at given x, y (None if no connection)."
        
        for obj in self.object_list:
            connection_list = obj.from_connections[:]
            connection_list.reverse()
            for con in connection_list:
                if (con.in_bounds(x, y)):
                    return con
        return None

    #   Object Methods

    def create_object(self, x, y) : 
        "Create a new object."
        self.add_object(self.object_tool.create_node(x, y))

    def add_object(self, obj) : 
        "Add a new object to the Canvas."
        
        self.object_list = [obj] + self.object_list

    def add_object_to_back(self, obj) : 
        "Add a new object to the Canvas at back of the list."
        
        self.object_list =  self.object_list + [obj]

    def remove_object(self, obj) : 
        "Remove an object from the canvas."
        
        for i in range(len(self.object_list)) : 
            if (obj == self.object_list[i]) : break
        else : return # object was not found
        del self.object_list[i]
        return i

    def toggle_object_density(self):
        if EditorSheet.show_density:
            EditorSheet.show_density = False
        else:
            EditorSheet.show_density = True
        self.refresh() 

    def toggle_object_activity(self):
        if EditorSheet.view == 'activity':
            EditorSheet.view = 'normal'
        else:
            EditorSheet.view = 'activity'
        self.refresh()

    def get_object_xy(self, x, y) : 
        "Return object at given x, y (or None if no object)."
        
        # search through the bounds of each object in the canvas. 
        # returns the first (nearest to front) object, None if no object at x,y
        for obj in self.object_list:
            if (obj.in_bounds(x, y)):
                break
        else : return None
        return obj


    def show_properties(self, focus):
        "Show properties of an object or connection, and remove the focus."
        
        if (focus != None):
            focus.show_properties()
            focus.set_focus(False)
        
    def delete_focus(self, focus):
        "Tell a connection or object to delete itself."
        
        if (focus == None) : 
            pass
        else:
            focus.remove()
            self.redraw_objects()

    #   Object Order Methods
    #
    # These methods ensure the ordering in the canvas window is held
    # and allows manipulation of the order.

    def redraw_objects(self, index = None):
        """
        Redraw all the objects in the canvas.

        If non-None, the index specifies that only the objects below
        that index need drawing.
        """

        if (index == None or index < 0) : index = len(self.object_list)
        for i in range(index ,0, -1):
            self.object_list[i-1].draw()
    
    def move_to_front(self, obj):
        index = self.remove_object(obj)
        self.add_object(obj)
        self.redraw_objects(index)

    def move_forward(self, obj):
        for i in range(len(self.object_list)) : # find object index in list
            if (obj == self.object_list[i]) : break
        else : return # object was not found
        # swap this object for the one higher in the canvas and redraw
        a = self.object_list[(i-1) : (i+1)]
        a.reverse()
        self.object_list[(i-1):(i+1)] = a
        self.redraw_objects(i+1)

    def move_to_back(self, obj):
        self.remove_object(obj)
        self.add_object_to_back(obj)
        self.redraw_objects()

    #   Hang List Methods
    #
    # If there is an object or connection at the right clicked point,
    # a popup menu is displayed, allowing for modifications to the
    # particular obj/con.

    def show_hang_list(self, event):

        # change to ARROW mode and get x, y mouse coords
        self.change_mode('a')
        x, y = self.canvasx(event.x), self.canvasy(event.y)
        # get connection at this point
        focus = self.get_connection_xy(x, y)
        for i in range(self.MAX_VIEWS) : # max number of views
            self.view.delete(END)
        if (focus == None):
            # if no connection, checks bounds of objects
            focus = self.get_object_xy(x, y)
            # fill in menu items that are just for objects
            for i in self.object_indices:
                self.item_menu.entryconfig(i, foreground = 'Black', activeforeground = 'Black')
        else:
            # gray out menu items that are just for objects
            for i in self.object_indices:
                self.item_menu.entryconfig(i,foreground = 'Gray', activeforeground = 'Gray')
        if (focus != None):
            for (label, function) in focus.viewing_choices:
                self.view.add_command(label = label, command = function)
            # give the connection or object the focus
            focus.set_focus(True)
            self.focus = focus
            # create the popup menu at current mouse coord
            self.item_menu.tk_popup(event.x_root, event.y_root)
        else:
            self.canvas_menu.tk_popup(event.x_root, event.y_root)
        
    #   Utility methods

    def save_snapshot(self):
        POSTSCRIPT_FILETYPES = [('Encapsulated PostScript images','*.eps'),
                                ('PostScript images','*.ps'),('All files','*')]
        snapshot_name = asksaveasfilename(filetypes=POSTSCRIPT_FILETYPES,
                                          initialdir=normalize_path(),
                                          initialfile=topo.sim.basename()+".ps")
        
        if snapshot_name:
            self.postscript(file=snapshot_name)

    def set_display_mode(self, mode):
        self.display_mode = mode
        for obj in self.object_list:
            obj.set_mode(mode)

    def set_tool_bars(self, arrow_tool, connection_tool, object_tool):
        # reference to the toolbar items, a tool is notified when the canvas is changed
        # to the mode corresponding to it.
        self.arrow_tool = arrow_tool
        # connection tool supplies 'connection' objects that can draw themselves in the canvas
        self.connection_tool = connection_tool
        # object tool supplies 'object' objects that can draw themselves in the canvas
        self.object_tool = object_tool
        # initialise mode
        self.change_mode('a')

    # does nothing
    def none(self, x, y) : pass

    def get_xy(self, x, y):
        "Returns the connection or object at this x, y position or None if there is not one."
        
        # check for a connection
        focus = self.get_connection_xy(x, y)
        if (focus == None):
            # if no connection, check bounds of objects
            focus = self.get_object_xy(x, y)
        return focus # return the first found or None




# JABALERT: I made this into parameterized.Parameterized to make self.warning work,
# but it should be changed to a PlotGroupPanel eventually
class ModelEditor(parameterized.Parameterized):
    """
    This class constructs the main editor window. It uses a instance
    of GUICanvas as the main editing canvas and inserts the
    three-option toolbar in a Frame along the left side of the window.
    """

    def __init__(self,master,**params):
        parameterized.Parameterized.__init__(self,**params)
        
        # create editor window and set title
        root = tk.AppWindow(master)
        root.title("Model Editor")

        canvas_frame = Frame(root,bg = 'white')
        canvas_frame.pack(side='right',fill = BOTH, expand = YES)

        toolbar_frame = Frame(root, bg = 'light grey', bd = 2)
        toolbar_frame.pack(side=LEFT,fill=Y) 

        self.canvas = EditorCanvas(canvas_frame)
        self.canvas.pack(fill = BOTH, expand = YES)     


        # object/node = sheet
        
        parameters_tool = ParametersTool(toolbar_frame)
        arrow_tool=ArrowTool(self.canvas,toolbar_frame,parameters_tool)
        object_tool=NodeTool(self.canvas,toolbar_frame,parameters_tool)
        connection_tool=ConnectionTool(self.canvas,toolbar_frame,parameters_tool)

        arrow_tool.pack(side='top')
        object_tool.pack(side='top')
        connection_tool.pack(side='top')
        parameters_tool.pack(side='top')


        # give the canvas a reference to the toolbars
        self.canvas.set_tool_bars(arrow_tool, connection_tool, object_tool) 

        # give the canvas focus and import any objects and connections already in the simulation
        self.canvas.focus_set()


        # Grid layout defaults
        self.xstart = 100
        self.ystart = 100
        self.next_x = self.xstart+100
        self.next_y = self.ystart
        self.xstep = 150
        self.ystep = 150

        self.import_model()

    def import_model(self):
        # get a list of all the objects in the simulation
        sim = self.canvas.simulation
        node_dictionary = sim.objects(EventProcessor)
        node_list = node_dictionary.values()

        # create the editor covers for the nodes
        for node in node_list:
            # if the sheet has x,y coords, use them
            if (hasattr(node,'layout_location') and node.layout_location!=(-1,-1)):
                x, y = node.layout_location
            # if not generate new coords on a grid layout
            # Could use dot/graphviz to place the objects nicely
            else:
                x, y = self.next_x , self.next_y
                self.next_y += self.ystep
                if self.next_y > canvas_region[3]:
                    self.next_y  = self.ystart
                    self.next_x += self.xstep
                if self.next_x > canvas_region[2]:
                    self.next_x = self.xstart + 15
                    self.next_y = self.ystart + 15

            # Could handle more EventProcessor subclasses here
            if isinstance(node,Sheet):
                editor_node = EditorSheet(self.canvas, node, (x, y), node.name)
            else:
                editor_node = EditorEP(self.canvas, node, (x, y), node.name)
                
            node.layout_location=(x,y)
            self.canvas.add_object(editor_node)

        # create the editor covers for the connections
        
        for editor_node in self.canvas.object_list:
            for con in editor_node.simobj.out_connections:
                # Could handle more EditorConnection subclasses here
                if isinstance(con,CFProjection):
                    editor_connection = EditorProjection("", self.canvas, editor_node)
                else:
                    editor_connection = EditorEPConnection("", self.canvas, editor_node)
                
                # find the EditorNode that the proj connects to
                for dest in self.canvas.object_list:
                    if (dest.simobj == con.dest):
                        # connect the connection to the destination node
                        editor_connection.connect(dest, con)
                        break
                else:
                    self.warning("The model editor cannot draw connection", con.name,
                                 "because", con.dest.name, "is not drawn in the editor.")
                           
                    
                    
        self.canvas.redraw_objects()























###########################################################################
## TOOLS
###########################################################################

class ArrowTool(Frame):
    """
    ArrowTool is a selectable frame containing an arrow icon and a label. It is a
    toolbar item in a ModelEditor that allows the user to change the GUICanvas to 
    'ARROW' mode. 
    """

    def __init__(self, canvas,  parent = None, parambar = None):
        Frame.__init__(self, parent,bg = 'light grey', bd = 4, relief = RAISED)
        self.canvas = canvas # hold canvas reference
        self.parameter_tool = parambar # To display class properties and name
        # label sets canvas mode
        self.title_label = Label(self, text="Move:", bg ='light grey')
        self.title_label.bind('<Button-1>', self.change_mode)
        self.title_label.pack()
        self.doc = 'Use the arrow tool to select and\nmove objects in the canvas around'
        # arrow icon
        self.icon = Canvas(self, width = 35, height = 30,bg = 'light grey')
        self.icon.create_polygon(10,0, 10,22, 16,17, 22,29, 33,22, 25,13, 33,8, 
            fill = 'black', outline = 'white')
        self.icon.pack()
        self.icon.bind('<Button-1>', self.change_mode) # icon sets canvas mode
        # pack in toolbar at top and fill out in X direction; click changes canvas mode
        self.pack(side = TOP, fill = X)
        self.bind('<Button-1>', self.change_mode)
        
        
    def change_mode(self, event):
        self.canvas.change_mode('a') # (ARROW)

    def set_focus(self, focus):
        "Change the background highlight to reflect whether this toolbar item is selected."
        
        if (focus):
             col = 'dark grey'; relief = GROOVE
             if not(self.parameter_tool == None):
                 self.parameter_tool.set_focus('Arrow', None, self.doc)
        else:
            col = 'light grey'; relief = RAISED

        # ALERT
        self.config(bg = col, relief = relief)
        self.title_label.config(bg = col)
        self.icon.config(bg = col)



# hack to sort by precedence
# (won't need when modeleditor is plotgroup)
def names_sorted_by_precedence(classes):
    classes.sort(lambda x, y: cmp(-x.precedence,-y.precedence))
    return [class_.__name__ for class_ in classes]
    

class NodeTool(Frame):
    """
    NodeTool extends Frame. It is expected to be included in a topographica
    model development GUI and functions as a self populating Node tool.
    The available Sheet types are supplied to be selected from. This Tool supplies
    a suitable Editor cover for a node and creates the corresponding topo object.
    """
    # hack: we can do this properly when converting to a plotgrouppanel
    default_sheet = 'CFSheet'
    
    def __init__(self, canvas,  parent = None, parambar = None):
        
        Frame.__init__(self, parent,bg = 'light grey', bd = 4, relief = RAISED)
        self.canvas = canvas # hold canvas reference.
        self.parameter_tool = parambar # To display class properties and name
        # bind clicks, pack in toolbar at top and fill out in X direction
        self.bind('<Button-1>', self.change_mode)
        self.pack(side = TOP, fill = X)
        # label sets canvas mode

        # CebAlerT: make these 'move' 'add sheet of type' things etc
        # look like buttons again, or -better - clean up the whole
        # interface!
        
        self.title_label = Label(self, text="Add sheet of type:",bg ='light grey')
        self.title_label.bind('<Button-1>', self.change_mode)
        self.title_label.pack()
        self.doc = 'Use the sheet tool to click a\nsheet object into the canvas.'
        # gets list of all the available sheets.
        self.sheet_list = param.concrete_descendents(Sheet)

        sheet_list = names_sorted_by_precedence(self.sheet_list.values())
        
        ## menu with list of available sheets
        self.option_var = StringVar()
        self.option_var.set(self.default_sheet)
        self.current_option = self.option_var.get()

        self.option_menu = Combobox(self,textvariable=self.option_var,
                                    values=sheet_list,state='readonly')
        self.option_menu.pack()
        self.option_menu.bind('<Button-1>', self.change_mode)
        
        self.option_var.trace_variable('w',self.set_option)
        
        



    #   Focus Methods
        
    def change_mode(self, option):
        self.canvas.change_mode('m') # ('MAKE')

    def set_focus(self, focus):
        "Change the background highlight to reflect whether this toolbar item is selected."
        
        if (focus):
            col = 'dark grey'; relief = GROOVE
            if not(self.parameter_tool == None):
                current_option = self.sheet_list[self.current_option]
                name = str(current_option).split('.')[-1][:-2]
                self.parameter_tool.set_focus(name, current_option, self.doc)
        else:
            col = 'light grey'; relief = RAISED

        # ALERT
        self.config(bg = col, relief = relief)
        self.title_label.config(bg = col)
        #self.option_menu.config(bg = col)


    #   Node Methods

    def create_node(self, x, y):
        if self.parameter_tool.focus:
            self.parameter_tool.update_parameters()
        # get the current selection and create the new topo object

        # CEBALERT: because Parameterized overwrites the name
        # unless it's passed in params when the object is created, I
        # pass the class name (set by ParametersFrameWithApply) here.
        # Same goes for projections. (i.e. Allow people to set the name
        # for a new sheet or projection.)
        name=self.sheet_list[self.current_option].name
        # Instead, should have a popup that asks for the name.

        if name:
            simobj = self.sheet_list[self.current_option](name=name)
        else:
            simobj = self.sheet_list[self.current_option]()
            
        sim = self.canvas.simulation # get the current simulation
        sim[simobj.name] = simobj
        # create the cover for the simobj and return it.
        return EditorSheet(self.canvas, simobj, (x, y), simobj.name)


    #   Util Methods
    
    def set_option(self,*args):
        """ """ 
        self.current_option = self.option_var.get()
        self.change_mode(None)




# JABHACKALERT: Currently only searches for topo.projection (connections have not been implemented yet).
class ConnectionTool(Frame):
    """ 
    ConnectionTool extends Frame. It is expected to be included in a topographica
    model development GUI and functions as a self populating Connection toolbar.
    The available Connection types are listed and the user can select one.
    When a connection is formed between two nodes a topo.projection of the
    specified type is instantiated and a reference to it is stored in it's Editor
    cover. Allows user to change the EditorCanvas mode to 'CONNECTION' mode.
    """

    def __init__(self, canvas, parent = None, parambar = None):
        # super constructor call.
        Frame.__init__(self, parent, bg = 'light grey', bd = 4, relief = RAISED)
        self.canvas = canvas # hold canvas reference.
        self.parameter_tool = parambar # To display class properties and name
        # bind clicks, pack in toolbar at top and fill out in X direction
        self.bind('<Button-1>', self.change_mode)
        self.pack(side = TOP, fill = X)
        # label sets canvas mode
        self.title_label = Label(self, text="Add projection of type:",bg ='light grey')
        self.title_label.bind('<Button-1>', self.change_mode)
        self.title_label.pack()
        self.doc = 'Use the connection tool to\ndrag connections between objects'
        # gets list of all the available projections.
        self.proj_list = param.concrete_descendents(Projection)
        proj_list = names_sorted_by_precedence(self.proj_list.values())


        ## menu with list of available projections
        self.option_var = StringVar()
        self.option_var.set(proj_list[0])
        self.current_option = self.option_var.get()
    
        self.option_menu = Combobox(self,textvariable=self.option_var,
                                    values=proj_list,state='readonly')

        self.option_menu.pack()
        self.option_menu.bind("<Button-1>",self.change_mode)
        
        self.option_var.trace_variable('w',self.set_option)



    #   Canvas Topo Linking Methods

    def new_cover(self, from_node):
        """
        Create an EditorProjection and return it.

        If there is more than one representation for connections/
        projections, the returned object will depend on the current
        selection.
        """
        return EditorProjection("", self.canvas, from_node)

    def create_connection(self, editor_connection, node):
        "Connects the editor connection and the topo simulation connection."

        if self.parameter_tool.focus:
            self.parameter_tool.update_parameters()
        sim = self.canvas.simulation
        from_node = editor_connection.from_node.simobj
        to_node = node.simobj
        con_type = self.proj_list[self.current_option]
        con_name = con_type.name
        # CEBHACKALERT: see alert about sheet name

        # CEBHACKALERT: should probably catch a specific error?
        try:
            if con_name is not None:
                con = sim.connect(from_node.name,to_node.name,connection_type=con_type,name=con_name)
            else:
                con = sim.connect(from_node.name,to_node.name,connection_type=con_type)
        except Exception, e:
            param.Parameterized().warning("Unable to connect these sheets with the given "+ self.current_option + " (" + str(e) +").")
            editor_connection.remove()
            return False
        
        editor_connection.connect(node, con)
        return True


    def set_option(self,*args):
        """ """ 
        self.current_option = self.option_var.get()
        self.change_mode(None)


    #   Focus Methods
    
    def change_mode(self, option):
        self.canvas.change_mode('c') # ('CONNECTION')

    def set_focus(self, focus):
        "Change the background highlight to reflect whether this toolbar item is selected."
        
        if (focus):
            col = 'dark grey'; relief = GROOVE
            if not(self.parameter_tool == None):
                current_option = self.proj_list[self.current_option]
                name = str(current_option).split('.')[-1][:-2]
                self.parameter_tool.set_focus(name, current_option, self.doc)
        else:
            col = 'light grey'; relief = RAISED

        # ALERT
        self.config(bg = col, relief = relief)
        self.title_label.config(bg = col)
        #self.option_menu.config(bg = col)


class ParametersTool(Frame):

    def __init__(self, parent = None):

        # CEBALERT: need to tidy up the title, positioning, etc.
        Frame.__init__(self, parent)
        self.focus = None
        # label
        self.title_label = Label(self)
        self.title_label.pack(side = TOP)
        self.doc_label = Label(self, font = ("Times", 12))
        self.doc_label.pack(side = TOP)

        # CEBALERT: will the users think they have to press 'apply' rather than just clicking
        # on the canvas to get the new object?
        self.parameter_frame = tk.ParametersFrameWithApply(self)#,buttons_to_remove=['Close','Defaults'])
        self.parameter_frame.hide_param('Close')
        self.parameter_frame.hide_param('Defaults')         
        self.parameter_frame.pack(side=BOTTOM)

    def update_parameters(self):
        self.parameter_frame.update_parameters()


    def set_focus(self, name, focus_class, doc = ''):
        
        self.focus = name
            
        self.title_label.config(text = name)
        self.doc_label.config(text = doc)

        if focus_class:
            self.parameter_frame.set_PO(focus_class)
            

        
###########################################################################
## OBJECTS
###########################################################################

# CEBALERT: should be a Parameterized
class EditorObject(object):
    """
    Anything that can be added and manipulated in an EditorCanvas. Every EditorCanvas
    has a corresponding Topo object associated with it. An instance of this class can 
    have the focus.
    """
    FROM = 0
    TO = 1

    def __init__(self, name, canvas,**params):
        self.canvas = canvas # retains a reference to the canvas
        self.name = name # set the name of the sheet
        self.focus = False # this does not have the focus
        self.viewing_choices = []
        self.label=None

    def draw(self):
        "Draw the object at the current x, y position."
        pass

    def objdoc(self):
        """Documentation string for this object."""
        ### JABALERT: Should be expanded to allow a per-object description,
        ### and should be bound to the actual editor object as well.
        return self.name + " is of type " + \
               shortclassname(self.simobj) + \
               ":\n\n" + str(getdoc(self.simobj))

    def show_properties(self):
        "Show parameters frame for object."
        
        parameter_window = tk.AppWindow(topo.guimain,status=True)
        #status=tk.StatusBar(parameter_window)
        #status.pack(side="bottom",fill='x',expand='yes')
        
        parameter_window.title(self.name)
        balloon = tk.Balloon(parameter_window)

#        import __main__;__main__.__dict__['AAA'] = parameter_window
        title = Label(parameter_window, text = self.name)
        title.pack(side = TOP)
        self.parameter_frame = tk.ParametersFrameWithApply(
            parameter_window,
            msg_handler=parameter_window.status)
        parameter_window.sizeright()
        
        balloon.bind(title,self.objdoc())
        self.parameter_window = parameter_window

    def update_parameters(self):
        self.parameter_frame.update_parameters()

    def okay_parameters(self, parameter_window):
        self.update_parameters()
        parameter_window.destroy()

    def set_focus(self, focus) : # set focus
        self.focus = focus

    def move(self):
        "Update position of object and redraw."
        pass

    def remove(self):
        "Remove this object from the canvas and from the Topographica simulation."
        pass    

    def in_bounds(self, x, y) : 
        "Return true if x,y lies within this gui object's boundary."
        pass



class EditorNode(EditorObject):
    """
    An EditorNode is used to cover any topographica node, presently this can only be a sheet.
    It is a sub class of EditorObject and supplies the methods required by any node to be used
    in a EditorCanvas. Extending classes will supply a draw method and other type specific 
    attributes. 
    """
    show_label= param.Boolean(default=True,
        doc="Whether to show a textual label for this object")

    def __init__(self, canvas, simobj, pos, name):
        EditorObject.__init__(self, name, canvas)
        self.from_connections = [] # connections from this node
        self.to_connections = [] # connections to this node
        self.x = pos[0] # set the x and y coords of the center of this node
        self.y = pos[1]
        self.mode = canvas.display_mode
        self.simobj = simobj

    #   Connection methods

    def attach_connection(self, con, from_to):
        if (from_to == self.FROM):
            if (con.from_node == con.to_node):
                self.from_connections = [con] + self.from_connections
            else:
                self.from_connections = self.from_connections + [con]
        else:
            if (con.from_node == con.to_node):
                self.to_connections = [con] + self.to_connections
            else:
                self.to_connections = self.to_connections + [con]
        
    def remove_connection(self, con, from_to) : # remove a connection to or from this node
        if (from_to):
            l = len(self.to_connections)
            for i in range(l):
                if (con == self.to_connections[i]) : break
            else : return
            del self.to_connections[i]
        else:
            l = len(self.from_connections)
            for i in range(l):
                if (con == self.from_connections[i]) : break
            else : return
            del self.from_connections[i]


    #   Util methods
    
    def get_pos(self):
        return (self.x, self.y) # return center point of node

    def show_properties(self):
        EditorObject.show_properties(self)
        self.parameter_frame.set_PO(self.simobj)
        Label(self.parameter_window, text = '\n\nConnections').pack(side = TOP)
        connections = list(set(self.to_connections).union(set(self.from_connections)))
        connection_list = [con.name for con in connections]

        self.connection_var = StringVar()
        self.connection_var.set(connection_list[0])
        
        connection_menu = OptionMenu(self.parameter_window,self.connection_var,*connection_list)

        self.connection_var.trace_variable('w',self.view_connection_parameters)

        connection_menu.pack(side = TOP)
        
    def view_connection_parameters(self, *args):
        con_sel = self.connection_var.get()
        for con in self.to_connections + self.from_connections:
            if con.name == con_sel:
                break
        else : 
            return
        con.show_properties()
        


# JABALERT: Should probably combine this with EditorNode
class EditorEP(EditorNode):
    """
    Represents any topo EventProcessor as a small, fixed-size oval by default.
    """

    def __init__(self, canvas, simobj, pos, name):
        EditorNode.__init__(self, canvas, simobj, pos, name)
        simobj.layout_location = (self.x,self.y) # store the ed coords in the topo sheet
        self.set_bounds()

        self.set_colours()
        col = self.colour[1]
        self.init_draw(col, False) # create a new parallelogram
        self.currentCol = col
        self.gradient = 1

    #   Draw methods

    def set_focus(self, focus):
        for id in self.id:
            self.canvas.delete(id)
        self.canvas.delete(self.label) # remove label   
        EditorNode.set_focus(self, focus) # call to super's set focus
        col = self.colour[not focus]
        self.init_draw(col, focus) # create new one with correct colour
        self.currentCol = col
        self.draw()

    def select_view(self, view_choice):
        self.view = view_choice
        self.set_focus(False)
        self.canvas.redraw_objects()

    def set_colours(self):
        colour = {'video':('dark red','black'),
                  'normal':('slate blue', 'lavender'),
                  'printing':('grey','white')}
        self.colour = colour[self.mode] # colours for drawing this node on the canvas

    def init_draw(self, colour, focus):
        if focus : label_colour = colour
        else : label_colour = 'black'
        h, w = 0.5*self.height, 0.5*self.width

        x, y = self.x, self.y
        x1,y1 = (x-w,y-h)
        x2,y2 = (x+w,y+h)

        self.id = [self.canvas.create_oval(x1,y1,x2,y2,fill=colour,outline="black")]
        dX = w + 5
        if (self.show_label):
            self.label = self.canvas.create_text(x - dX, y, anchor = E, fill = label_colour, text = self.name)

    def dec_to_hex_str(self, val, length):
        # expects a normalised value and maps it to a hex value of the given length
        max_val = pow(16, length) - 1
        fmt = '%%0%dx'%length
        return fmt % (val*max_val)

    def draw(self, x = 0, y = 0):
        # move the parallelogram and label by the given x, y coords (default redraw)
        if not(x == y == 0):
            for id in self.id:
                self.canvas.move(id, x, y)
            if (self.show_label):
                self.canvas.move(self.label, x, y)
        for id in self.id:
            self.canvas.tag_raise(id)
        if (self.show_label):
            self.canvas.tag_raise(self.label)
        # redraw the connections
        for con in self.to_connections : 
            if (con.from_node == con.to_node):
                con.move()
        for con in self.to_connections:
            if (not(con.from_node == con.to_node)):
                con.move()
        for con in self.from_connections:
            if (not(con.from_node == con.to_node)):
                con.move()


    #   Update methods

    def remove(self):
        l = len(self.from_connections) # remove all the connections from and to this sheet
        for index in range(l):
            self.from_connections[0].remove()
        l = len(self.to_connections)
        for index in range(l):
            self.to_connections[0].remove()
        for id in self.id:
            self.canvas.delete(id)
        self.canvas.delete(self.label)
        self.canvas.remove_object(self) # remove from canvas' object list
        del topo.sim[self.simobj.name] # actually delete the sheet

    def move(self, x, y):
        # the connections position is updated
        old = self.x, self.y
        self.x = x
        self.y = y
        self.simobj.layout_location = (self.x,self.y) # update topo sheet position
        self.draw(self.x - old[0], self.y - old[1])


    #   Connection methods

    def remove_connection(self, con, from_to):
        EditorNode.remove_connection(self, con, from_to)
        if from_to:
            node = con.from_node
        else:
            node = con.to_node
        index = con.draw_index
        # Decrease the indexes of the connections between the same nodes and with a higher index.
        for connection in self.from_connections:
            if (node == connection.to_node and connection.draw_index >= index):
                connection.decrement_draw_index()
            if connection.to_node == connection.from_node : return
        for connection in self.to_connections :       
            if (node == connection.from_node and connection.draw_index >= index):
                connection.decrement_draw_index()

    def get_connection_count(self, node):
        count = 0
        for con in self.from_connections:
            if (con.to_node == node):
                count += 1
        for con in self.to_connections:
            if (con.from_node == node):
                count += 1
        if node == self : count /= 2
        return count


    #   Util methods

    def in_bounds(self, pos_x, pos_y):
        return ((pos_x> self.x-self.width)  and (pos_x<= self.x+self.width) and 
                (pos_y> self.y-self.height) and (pos_y<= self.y+self.height))


    def set_bounds(self):
        # Default representation: fixed-size node
        self.width=15
        self.height=self.width*0.7


    def set_mode(self, mode):
        self.mode = mode
        self.set_colours()

        for id in self.id:
            self.canvas.delete(id)
        self.canvas.delete(self.label) # remove label
        self.init_draw(self.colour[not self.focus], self.focus)
        for con in self.to_connections:
            con.set_mode(mode)
        for con in self.from_connections:
            con.draw()




class EditorSheet(EditorEP):
    """
    Represents any topo sheet. It is a subclass of EditorEP and fills in the
    methods that are not defined. It is represented by a Parallelogram in its
    Canvas. The colours used for drawing can be set. Uses bounding box to
    determine if x, y coord is within its boundary.
    """
    normalize = param.Boolean(default=False)
    show_density = param.Boolean(default=False)
    view = param.ObjectSelector(default='activity',objects=['normal','activity'])
    
    def __init__(self, canvas, simobj, pos, name):
        # Should call EditorEP's constructor instead
        EditorNode.__init__(self, canvas, simobj, pos, name)
        simobj.layout_location = (self.x,self.y) # store the ed coords in the topo sheet
        self.element_count = self.matrix_element_count()
        self.set_bounds()

        self.set_colours()
        col = self.colour[1]
        self.init_draw(col, False) # create a new parallelogram
        self.currentCol = col
        self.gradient = 1
        self.viewing_choices = [('Normal', lambda: self.select_view('normal')),
                                ('Activity', lambda: self.select_view('activity'))]


    #   Draw methods

    def set_focus(self, focus):
        for id in self.id:
            self.canvas.delete(id)
        self.canvas.delete(self.label) # remove label   
        EditorNode.set_focus(self, focus) # call to super's set focus
        col = self.colour[not focus]
        self.init_draw(col, focus) # create new one with correct colour
        self.currentCol = col
        self.draw()

    def init_draw(self, colour, focus):
        self.id = []
        if focus : label_colour = colour
        else : label_colour = 'black'
        factor = self.canvas.scaling_factor
        h, w = 0.5 * self.height * factor, 0.5 * self.width *factor
        if not(self.focus):
            if self.view == 'activity':
                colour = ''
                x, y = self.x - w + h, self.y - h
                # AL, the idea will be to allow any available plots to be shown on the sheet.
                # eg m = self.simobj.sheet_views['OrientationPreference'].view()[0]
                update_activity()
                m = self.simobj.sheet_views['Activity'].view()[0]
                if self.normalize == True:
                    m = self.normalize_plot(m)
                matrix_width, matrix_height = self.element_count
                dX, dY = (w * 2)/ matrix_width, (h * 2) / matrix_height
                for i in range(matrix_height):
                    for j in range(matrix_width):
                        a = i * dY
                        x1, y1 = x - a + (j * dX), y + a
                        x2, y2 = x1 - dY, y1 + dY
                        x3, x4 = x2 + dX, x1 + dX
                        point = m[i][j]
                        if point < 0 : point = 0.0
                        if point > 1 : point = 1.0
                        col = '#' + (self.dec_to_hex_str(point, 3)) * 3
                        self.id = self.id + [self.canvas.create_polygon
                           (x1, y1, x2, y2, x3, y2, x4, y1, fill = col, outline = col)]
        x, y = self.x, self.y
        x1,y1 = (x - w - h, y + h)
        x2,y2 = (x - w + h, y - h)
        x3,y3 = x2 + (w * 2), y2
        x4,y4 = x1 + (w * 2), y1
        self.id = self.id + [self.canvas.create_polygon(x1, y1, x2, y2, x3, y3, x4, y4, 
            fill = colour , outline = "black")]
        dX = w + 5
        if (self.show_label):
            self.label = self.canvas.create_text(x - dX, y, anchor = E, fill = label_colour, text = self.name)
        # adds a density grid over the sheet
        if self.show_density:
            x, y = self.x - w + h, self.y - h
            matrix_width, matrix_height = self.element_count
            dX, dY = (w * 2)/ matrix_width, (h * 2) / matrix_height
            for i in range(matrix_height + 1):
                x1 = x - (i * dY)
                x2 = x1 + (w * 2)
                y1 = y + (i * dY)
                self.id = self.id + [self.canvas.create_line(x1, y1, x2, y1, fill = 'slate blue')]
            for j in range(matrix_width + 1):
                x1 = x + (j * dX)
                x2 = x1 - (h * 2)
                y1 = y
                y2 = y1 + (h * 2)
                self.id = self.id + [self.canvas.create_line(x1, y1, x2, y2, fill = 'slate blue')]

    def normalize_plot(self,a):
        """ 
        Normalize an array s.
        In case of a constant array, ones is returned for value greater than zero,
        and zeros in case of value inferior or equal to zero.
        """
        # AL is it possible to use the normalize method in plot?
        from numpy.oldnumeric import zeros, ones, Float, divide
        a_offset = a-min(a.ravel())
        max_a_offset = max(a_offset.ravel())
        if max_a_offset>0:
             a = divide(a_offset,float(max_a_offset))
        else:
             if min(a.ravel())<=0:
                  a=zeros(a.shape,Float)
             else:
                  a=ones(a.shape,Float)
        return a

    #   Util methods

    def in_bounds(self, pos_x, pos_y) : # returns true if point lies in a bounding box
        # if the coord is pressed within the parallelogram representation this 
        # returns true.
        # Get the parallelogram points and centers them around the given point
        x = self.x - pos_x; y = self.y - pos_y
        w = 0.5 * self.width * self.canvas.scaling_factor
        h = 0.5 * self.height * self.canvas.scaling_factor
        A = (x - w - h, y + h)
        B = (x - w + h, y - h)
        C = B[0] + (2 * w), B[1]
        D = A[0] + (2 * w), A[1]
        # calculate the line constants  
        # As the gradient of the lines is 1 the calculation is simple.
        a_AB = A[1] + A[0]
        a_CD = C[1] + C[0]
        # The points are centered around the given coord, finding the
        # intersects with line y = 0 and ensuring that the left line
        # lies on the negative side of the point and the right line
        # lies on the positive side of the point determines that the
        # point is within the parallelogram.
        if ((D[1] >= 0) and (B[1] <= 0) and (a_AB <= 0) and (a_CD >= 0)):
            return True
        return False


    def matrix_element_count(self):
        # returns the length and width of the matrix that holds this sheet's plot values
        l,b,r,t = self.simobj.bounds.aarect().lbrt()
        density = self.simobj.xdensity
        return int(density * (r - l)), int(density * (t - b))
    
    def set_bounds(self):
        # Use the default sheet bounds as to set the "normal" size
        # of SheetObject in the GUI, so simulations using very large
        # sheets still look normal.
        dl,db,dr,dt = self.simobj.__class__.nominal_bounds.aarect().lbrt()
        width_fact = 120.0 / (dr - dl) 
        height_fact = 60.0 / (dt - db)
        l,b,r,t = self.simobj.bounds.aarect().lbrt()
        self.width = width_fact * (r - l) * self.canvas.scaling_factor
        self.height = height_fact * (t - b) * self.canvas.scaling_factor




class EditorConnection(EditorObject):

    """
    A connection formed between 2 EditorNodes on a EditorCanvas. A EditorConnection is used
    to cover any topographica connection (connection / projection), and extending 
    classes will supply a draw method and other type specific attributes.
    """

    show_label= param.Boolean(default=True,
        doc="Whether to show a textual label for this object")

    def __init__(self, name, canvas, from_node):
        EditorObject.__init__(self, name, canvas)
        self.from_node = from_node # initial node selected
        self.to_node = None # updated when the user selects the second node - self.connect(..)
        # temporary point, for when the to connection node is undefined
        self.to_position = from_node.get_pos()
        self.mode = canvas.display_mode


    #   Draw methods

    def set_focus(self, focus) : # give this connection the focus
        EditorObject.set_focus(self, focus)
        self.draw()


    #   Update methods
    
    def move(self):
        # if one of the nodes connected by this connection move, then move by redrawing 
        self.draw() 

    def update_position(self, pos) : # update the temporary point
        self.to_position = pos
        self.draw()

    def connect(self, to_node, con) : # pass the node this connection is to
        self.simobj = con
        if (self.name == ""):
            self.name = con.name
        self.to_node = to_node # store a reference to the node this is connected to
        self.to_position = None
        self.from_node.attach_connection(self, self.FROM) # tell the sheets that they are connected.
        self.to_node.attach_connection(self, self.TO)

    def remove(self):
        # CEBALERT: there's no code here to handle GUI object
        # removal (though the EditorProjection subclass does have GUI
        # removal code, so presumably projections do get removed from
        # the screen. But I'm confused about what is treated as a
        # connection, and what as a projection in the model editor).
        if hasattr(self,'simobj'):
            self.simobj.remove()


    #   Util methods
    
    def show_properties(self):
        EditorObject.show_properties(self)
        self.parameter_frame.set_PO(self.simobj)


# JABALERT: Should probably combine this with EditorConnection
class EditorEPConnection(EditorConnection):
    """
    Represents any topo EPConnection using a line with an arrow head in the middle.
    """ 

    def __init__(self, name, canvas, from_node):
        EditorConnection.__init__(self, name, canvas, from_node)
        # if more than one connection between nodes, 
        # this will reflect how to draw this connection
        self.draw_index = 0
        self.deviation = 0
        self.gradient = (1,1)
        self.id = (None,None)
        self.balloon = tk.Balloon(canvas)
        self.set_colours()
        self.view = 'line'
        self.draw_fn = self.draw_line
        self.viewing_choices = [('Line', lambda: self.select_view('line'))]
        self.update_factor()

    #   Draw methods
    
    def select_view(self, view_choice):
        self.view = view_choice
        self.move()
        self.set_focus(False)  

    def set_colours(self):
        colours = {'video' : ('dark red', 'blue', 'yellow'),
                   'normal': ('dark red', 'blue', 'yellow'),
                   'printing': ('grey', 'black', 'black')}
        self.colour = colours[self.mode]

    def draw(self):
        # determine if connected to a second node, and find the correct from_position
        for id in self.id : # remove the old connection
            self.canvas.delete(id)
        self.canvas.delete(self.label)
        from_position = self.from_node.get_pos() # get the center points of the two nodes
        if (self.to_node == None) :  # if not connected yet, use temporary point.
            to_position = self.to_position
        else:
            to_position = self.to_node.get_pos()

        self.draw_fn(from_position, to_position)

    def draw_line(self,from_position, to_position):
        # set the colour to be used depending on whether connection has the focus.
        if (self.focus) : 
            text_col = col = self.colour[0]
        else:
            text_col = 'black'
            col = self.colour[1]
        middle = self.get_middle(from_position, to_position)
        factor = self.canvas.scaling_factor
        if (to_position == from_position) : # connection to and from the same node
            deviation = self.draw_index * 15 * factor
            x1 = to_position[0] - ((20 * factor) + deviation)
            y2 = to_position[1]
            x2 = x1 + (40 * factor) + (2 * deviation)
            y1 = y2 - ((30 * factor) + deviation) 
            midX = self.get_middle((x1,0),(x2,0))[0]
            # create oval and an arrow head.
            self.id = (self.canvas.create_oval(x1, y1, x2, y2, outline = col), 
            self.canvas.create_line(midX, y1, midX+1, y1, arrow = FIRST, fill = col))
            # draw name label beside arrow head
            if (self.show_label):
                self.label = self.canvas.create_text(middle[0] - 
                    (20 + len(self.name)*3), middle[1] - (30 + deviation) , text = self.name)
        else :    
            # create a line between the nodes - use 2 to make arrow in center.
            dev = self.deviation
            from_pos = from_position[0] + self.deviation, from_position[1]
            mid = middle[0] + 0.5 * dev, middle[1]
            self.id = (self.canvas.create_line(from_pos, mid , arrow = LAST, fill = col),
                    self.canvas.create_line(mid, to_position, fill = col))
            # draw name label
            dX = 20 * factor
            dY = self.draw_index * 20 * factor
            if (self.show_label):
                self.label = self.canvas.create_text(middle[0] - dX,
                    middle[1] - dY, fill = text_col, text = self.name, anchor = E)


    #   Update methods
    def remove(self):
        if (self.to_node != None) : # if a connection had been made then remove it from the 'to' node
            self.to_node.remove_connection(self, self.TO)
            self.from_node.remove_connection(self, self.FROM) # and remove from 'from' node
        for id in self.id : # remove the representation from the canvas
            self.canvas.delete(id)
        self.canvas.delete(self.label)

        # CEBALERT: see earlier alert about EditorObject not inheriting from object.
        EditorConnection.remove(self) #super(EditorProjection,self).remove()
        

    def move(self):
        # if one of the nodes connected by this connection move, then move by redrawing 
        self.gradient = self.calculate_gradient()
        self.update_factor()
        self.draw() 

    def decrement_draw_index(self):
        self.draw_index -= 1
        if self.to_node == self.from_node:
            self.update_factor()
        else: 
            self.connect_to_coord((self.from_node.width / 2) - 10)

    def connect(self, to_node, con):
        EditorConnection.connect(self, to_node, con)
        self.draw_index = self.from_node.get_connection_count(to_node)-1
        if (self.from_node == to_node):
            self.update_factor() 
        else:
            self.connect_to_coord((self.from_node.width / 2) - 10)
        self.gradient = self.calculate_gradient()
        self.radius = self.get_radius()

    #   Util methods
    
    def get_middle(self, pos1, pos2) : # returns the middle of two points
        return (pos1[0] + (pos2[0] - pos1[0])*0.5, pos1[1] + (pos2[1] - pos1[1])*0.5)

    def get_radius(self):
        """Not implemented in this class"""
        return (0,0)

    def update_factor(self):
        pass

    def connect_to_coord(self, width):
        n = self.draw_index
        sign = math.pow(-1, n)
        self.deviation = sign * width + (-sign) * math.pow(0.5, math.ceil(0.5 * (n))) * width

    # returns the gradients of the two lines making the opening 'v' part of the receptive field. 
    # this depends on the draw_index, as it determines where the projection's representation begins.
    def calculate_gradient(self):
        """Not implemented in this class"""
        return (1,1)

    def in_bounds(self, x, y) : # returns true if point lies in a bounding box
        factor = self.canvas.scaling_factor
        # If connections are represented as lines
        # currently uses an extent around the arrow head.
        to_position = self.to_node.get_pos()
        from_position = self.from_node.get_pos()
        if (self.to_node == self.from_node):
            dev = self.draw_index * 15 * factor
            middle = (to_position[0], to_position[1] - ((30 * factor) + dev))
        else:
            dev = self.deviation * 0.5
            middle = self.get_middle(from_position, to_position)
        if ((x < middle[0] + 10 + dev) & (x > middle[0] - 10 + dev) & (y < middle[1] + 10) & (y > middle[1] - 10)):
            return True
        return False

    def set_mode(self, mode):
        self.mode = mode
        self.set_colours()
        self.draw()



class EditorProjection(EditorEPConnection):
    """
    Represents any topo CFProjection. It is a subclass of EditorEPConnection and fills 
    in the methods that are not defined. Can be represented by a representation of a
    projection's receptive field or by a line with an arrow head in the middle;
    lateral projections are represented by a dotted ellipse around the center.
    Can determine if x,y coord is within the triangular receptive field or within an
    area around the arrow head. The same can be determined for a lateral projection 
    ellipse.
    """
    
    def __init__(self, name, canvas, from_node, receptive_field = True):
        EditorEPConnection.__init__(self, name, canvas, from_node)
        # if more than one connection between nodes, 
        # this will reflect how to draw this connection
        self.normal_radius = 15
        self.radius = self.get_radius()
        self.receptive_field = receptive_field
        self.view = 'radius'
        self.viewing_choices = [('Field Radius', lambda: self.select_view('radius')),
                                ('Line', lambda: self.select_view('line')),
                                ('Fixed Size', lambda: self.select_view('normal'))]


    #   Draw methods

    def draw(self):
        self.draw_fn = \
        {'normal' : self.draw_normal,
         'line' :   self.draw_line,
         'radius' : self.draw_radius
        }[self.view]
        EditorEPConnection.draw(self)


    def draw_radius(self, from_position, to_position):
         # set the colour to be used depending on whether connection has the focus.
        if (self.focus) : 
            text_col = col = self.colour[0]
            lateral_colour = self.from_node.colour[0]
        else:
            text_col = 'black'
            col = self.colour[1]
            lateral_colour = ''
        # midpoint of line
        middle = self.get_middle(from_position, to_position)
        if (to_position == from_position) : # connection to and from the same node
            a, b = self.get_radius()
            x1 = to_position[0] - a
            y1 = to_position[1] + b
            x2 = to_position[0] + a
            y2 = to_position[1] - b
            self.id = (self.canvas.create_oval(x1, y1, x2, y2, fill = lateral_colour, 
                dash = (2,2), outline = self.colour[2], width = 2), None)

# CEBALERT: as far as I know, this balloon binding never worked.
#            self.balloon.tagbind(self.canvas, self.id[0], self.name)       

        else :  # connection between distinct nodes
            x1, y1 = to_position
            x2, y2 = from_position
            # this is for cases when the radius changes size.
            radius_x, radius_y = self.get_radius()
            self.id = (self.canvas.create_line(x1, y1, x2 - radius_x, y2, fill = col),
                self.canvas.create_line(x1, y1, x2 + radius_x, y2, fill = col),
                self.canvas.create_oval(x2 - radius_x, y2 - radius_y, 
                    x2 + radius_x, y2 + radius_y, outline = col))
            # draw name label
            dX = 20
            dY = self.draw_index * 20
            if (self.show_label):
                self.label = self.canvas.create_text(middle[0] - dX,
                    middle[1] - dY, fill = text_col, text = self.name, anchor = E)


    def draw_normal(self, from_position, to_position):
        # set the colour to be used depending on whether connection has the focus.
        if (self.focus) : 
            text_col = col = self.colour[0]
            lateral_colour = self.from_node.colour[0]
        else:
            text_col = 'black'
            col = self.colour[1]
            lateral_colour = ''
        # midpoint of line
        middle = self.get_middle(from_position, to_position)
        if (to_position == from_position) : # connection to and from the same node
            a, b = self.factor
            x1 = to_position[0] - a
            y1 = to_position[1] + b
            x2 = to_position[0] + a
            y2 = to_position[1] - b
            self.id = (self.canvas.create_oval(x1, y1, x2, y2, fill = lateral_colour, 
                dash = (2,2), outline = self.colour[2], width = 2), None)

#            self.balloon.tagbind(self.canvas, self.id[0], self.name)

        else :  # connection between distinct nodes
            x1, y1 = to_position
            x2, y2 = from_position
            x2 += self.deviation
            radius = self.normal_radius
            self.id = (self.canvas.create_line(x1, y1, x2 - radius, y2, fill = col),
                self.canvas.create_line(x1, y1, x2 + radius, y2, fill = col),
                self.canvas.create_oval(x2 - radius, y2 - (0.5 * radius), 
                x2 + radius, y2 + (0.5 * radius), outline = col))
            # draw name label
            dX = 20
            dY = self.draw_index * 20
            if (self.show_label):
                self.label = self.canvas.create_text(middle[0] - dX,
                    middle[1] - dY, fill = text_col, text = self.name, anchor = E)
        

    #   Util methods
    
    def get_radius(self):
        factor = self.canvas.scaling_factor
        if self.to_node == None:
            return (factor * self.normal_radius, factor * self.normal_radius * 
                self.from_node.height / self.from_node.width)
        node = self.from_node
        node_bounds = node.simobj.bounds.aarect().lbrt()

        try:
            bounds = self.simobj.bounds_template.lbrt()
        except AttributeError:
            return (factor * self.normal_radius, factor * self.normal_radius * 
                self.from_node.height / self.from_node.width)
        radius_x = factor * (node.width / 2) * (bounds[2] - bounds[0]) / (node_bounds[2] - node_bounds[0])
        radius_y = radius_x * node.height / node.width
        return radius_x, radius_y

    # returns the size of the semimajor and semiminor axis of the ellipse representing 
    # the draw_index-th lateral projection.
    def get_factor(self):
        factor = self.canvas.scaling_factor
        w = factor * self.from_node.width; h = factor * self.from_node.height
        a = 20; n = (w / 2) - 10; b = (n - a)
        major = a + (b * (1 - pow(0.8, self.draw_index)))
        a = 20 * h / w; n = (h / 2) - 10; b = (n - a)
        minor = a + (b * (1 - pow(0.8, self.draw_index)))
        return major, minor

    def update_factor(self):
        self.factor = self.get_factor()

    # returns the gradients of the two lines making the opening 'v' part of the receptive field. 
    # this depends on the draw_index, as it determines where the projection's representation begins.
    def calculate_gradient(self):
        factor = self.canvas.scaling_factor
        if self.view == 'radius':
            A = self.to_node.get_pos()
            T = self.from_node.get_pos()
            B = (T[0] - self.radius[0], T[1])
            C = (T[0] + self.radius[0], T[1])
        else:
            A = self.to_node.get_pos()
            T = (self.from_node.get_pos()[0] + self.deviation ,self.from_node.get_pos()[1])
            B = (T[0] - (factor * self.normal_radius), T[1])
            C = (T[0] + (factor * self.normal_radius), T[1])
        den_BA = (A[0] - B[0])
        if not(den_BA == 0):
            m_BA = (A[1] - B[1]) / den_BA
        else : 
            m_BA = 99999 # AL - this should be a big number
        den_CA = (A[0] - C[0])
        if not(den_CA == 0):
            m_CA = (A[1] - C[1]) / den_CA
        else : 
            m_CA = 99999 # AL - this should be a big number
        return (m_BA, m_CA)

    def in_bounds(self, x, y) : # returns true if point lies in a bounding box
        factor = self.canvas.scaling_factor
        if self.view == 'line':
            return EditorEPConnection.in_bounds(self,x,y)
        else:
            # returns true if x, y lie inside the oval representing this lateral projection
            if (self.to_node == None or self.to_node == self.from_node):
                if self.view == 'radius':
                    a, b = self.get_radius()
                else:
                    a, b = self.factor
                x, y = x - self.to_node.get_pos()[0], y - self.to_node.get_pos()[1]
                if (x > a or x < -a):
                    return False
                pY = math.sqrt(pow(b,2) * (1 - (pow(x,2)/pow(a,2))))
                if (y > pY or y < -pY):
                    return False
                return True
            # returns true if x, y lie inside the triangular receptive
            # field representing this projection get the points of the
            # triangular receptive field, centered around the x, y
            # point given
            to_position = self.to_node.get_pos()
            from_position = self.from_node.get_pos()
            if self.view == 'radius':
                A = (to_position[0] - x, to_position[1] - y)
                T = (from_position[0] + (self.deviation * factor) - x, from_position[1] - y)
                B = (T[0] - self.radius[0], T[1])
                #C = (T[0] + self.radius[0], T[1])
            else:
                A = (to_position[0] - x, to_position[1] - y)
                T = (from_position[0] + (self.deviation * factor) - x, from_position[1] - y)
                B = (T[0] - (self.normal_radius * factor), T[1])
                #C = (T[0] + (self.normal_radius * factor), T[1])
            # if the y coords lie outwith the boundaries, return false
            if (((A[1] < B[1]) and (B[1] < 0 or A[1] > 0)) or 
                ((A[1] >= B[1]) and (B[1] > 0 or A[1] < 0))):
                return False
            # calculate the constant for the lines of the triangle
            a_BA = A[1] - (self.gradient[0] * A[0])
            a_CA = A[1] - (self.gradient[1] * A[0])
            # The points are centered around the given coord, finding
            # the intersects with line y = 0 and ensuring that the
            # left line lies on the negative side of the point and the
            # right line lies on the positive side of the point
            # determines that the point is within the triangle.
            if (((0 - a_CA) / self.gradient[1] >= 0) and ((0 - a_BA) / self.gradient[0] <= 0)):
                return True
            return False



"""
Extensions to pickle allowing items in __main__ to be saved.
"""

# CEBALERT: move into snapshots.py?

import new
import pickle
import types
import __main__
from StringIO import StringIO

import copy

def _name_is_main(obj):
    # CEBALERT: see IPython hack in commandline.py
    return obj.__module__ == "__main__" or obj.__module__ == "__mynamespace__"

    

class PickleMain(object):
    """
    Pickle support for types and functions defined in __main__.
    
    When pickled, saves types and functions defined in __main__ by
    value (i.e. as bytecode). When unpickled, loads previously saved
    types and functions back into __main__.
    """
    def _create_pickler(self):
        # Usually we use the cPickle module rather than the pickle
        # module (because cPickle is faster), but here we need control
        # over the pickling process so we use pickle.
        #
        # Additionally, we create a Pickler instance to avoid changing
        # defaults in the pickle module itself, so that there are no side
        # effects for code elsewhere (although we don't use pickle
        # anywhere else ourselves...).

        self.pickled_bytecode = StringIO()
        self.pickler = pickle.Pickler(self.pickled_bytecode,-1)

        # CB: pickle.Pickler's dispatch attribute is a class
        # attribute (I don't know why, but it is...), so before
        # modifying this instance's dispatch attribute we make sure
        # modifications will affect only this instance.
        self.pickler.dispatch = copy.copy(self.pickler.dispatch)

        self.pickler.dispatch[new.code] = save_code
        self.pickler.dispatch[new.function] = save_function
        self.pickler.dispatch[dict] = save_module_dict
        self.pickler.dispatch[new.classobj] = save_classobj
        self.pickler.dispatch[new.instancemethod] = save_instancemethod
        self.pickler.dispatch[new.module] = save_module
        self.pickler.dispatch[type] = save_type

        # CB: maybe this should be registered from elsewhere
        import param
        self.pickler.dispatch[param.parameterized.ParameterizedMetaclass] = save_type


    def __getstate__(self):
        self._create_pickler()
        
        bytecode = {}
        for name,obj in __main__.__dict__.items():
            if not name.startswith('_'):
                if isinstance(obj,types.FunctionType) or isinstance(obj,type):
                    # (could be extended to other types, I guess
                    if _name_is_main(obj): 
                        #CB: how do I print out info via Parameterized?
                        print "%s is defined in __main__: saving bytecode."%name
                        bytecode[name] = obj

        self.pickler.dump(bytecode)
        return {'pickled_bytecode':self.pickled_bytecode}


    def __setstate__(self,state):
        bytecode = pickle.load(StringIO(state['pickled_bytecode'].getvalue()))

        for name,obj in bytecode.items():
            print "%s restored from bytecode into __main__"%name
            __main__.__dict__[name] = obj





### Copied from http://code.activestate.com/recipes/572213/ ###

# Original docstring
#
# Extend pickle module to allow pickling of interpreter state
# including any interactively defined functions and classes.
#
# This module is not required for unpickling such pickle files.
#
# >>> import savestate, pickle, __main__
# >>> pickle.dump(__main__, open('savestate.pickle', 'wb'), 2)

# (note that we're not actually using it to pickle __main__, and I've
# removed the lines that change pickle's defaults)


def save_code(self, obj):
    """ Save a code object by value """
    args = (
        obj.co_argcount, obj.co_nlocals, obj.co_stacksize, obj.co_flags, obj.co_code,
        obj.co_consts, obj.co_names, obj.co_varnames, obj.co_filename, obj.co_name,
        obj.co_firstlineno, obj.co_lnotab, obj.co_freevars, obj.co_cellvars
    )
    self.save_reduce(new.code, args, obj=obj)

def save_function(self, obj):
    """ Save functions by value if they are defined interactively """
    if _name_is_main(obj) or obj.func_name == '<lambda>':
        args = (obj.func_code, obj.func_globals, obj.func_name, obj.func_defaults, obj.func_closure)
        self.save_reduce(new.function, args, obj=obj)
    else:
        self.save_global(obj)
        #pickle.Pickler.save_global(self, obj)

def save_global_byname(self, obj, modname, objname):
    """ Save obj as a global reference. Used for objects that pickle does not find correctly. """
    self.write('%s%s\n%s\n' % (pickle.GLOBAL, modname, objname))
    self.memoize(obj)

def save_module_dict(self, obj, main_dict=vars(__import__('__main__'))):
    """ Special-case __main__.__dict__. Useful for a function's func_globals member. """
    if obj is main_dict:
        save_global_byname(self, obj, '__main__', '__dict__')
    else:
        return self.save_dict(obj)
        #return pickle.Pickler.save_dict(self, obj)      # fallback to original 

def save_classobj(self, obj):
    """ Save an interactively defined classic class object by value """
    if _name_is_main(obj):
        args = (obj.__name__, obj.__bases__, obj.__dict__)
        self.save_reduce(new.classobj, args, obj=obj)
    else:  
        name = str(obj).split('.')[-1]  # CEB: hack to find classic class name
        self.save_global(obj,name)
        #pickle.Pickler.save_global(self, obj, name)

def save_instancemethod(self, obj):
    """ Save an instancemethod object """
    # Instancemethods are re-created each time they are accessed so this will not be memoized
    args = (obj.im_func, obj.im_self, obj.im_class)
    self.save_reduce(new.instancemethod, args)

def save_module(self, obj):
    """ Save modules by reference, except __main__ which also gets its contents saved by value """
    if _name_is_main(obj):
        self.save_reduce(__import__, (obj.__name__,), obj=obj, state=vars(obj).copy())
    elif obj.__name__.count('.') == 0:
        self.save_reduce(__import__, (obj.__name__,), obj=obj)
    else:
        save_global_byname(self, obj, *obj.__name__.rsplit('.', 1))

def save_type(self, obj):
    if getattr(new, obj.__name__, None) is obj:
        # Types in 'new' module claim their module is '__builtin__' but are not actually there
        save_global_byname(self, obj, 'new', obj.__name__)
    elif _name_is_main(obj):
        # Types in __main__ are saved by value

        # Make sure we have a reference to type.__new__        
        if id(type.__new__) not in self.memo:
            self.save_reduce(getattr, (type, '__new__'), obj=type.__new__)
            self.write(pickle.POP)

        # Copy dictproxy to real dict
        d = dict(obj.__dict__)
        # Clean up unpickleable descriptors added by Python
        d.pop('__dict__', None)
        d.pop('__weakref__', None)
        
        args = (type(obj), obj.__name__, obj.__bases__, d)
        self.save_reduce(type.__new__, args, obj=obj)
    else:
        # Fallback to default behavior: save by reference
        self.save_global(obj)
        #pickle.Pickler.save_global(self, obj)
###############################################################        



"""
Learning functions and projection-level learning functions (see projfn.py)
written in C to optimize performance.

Requires the weave package; without it unoptimized versions are used.

$Id$
"""

from numpy import zeros, ones

import param

from topo.base.sheet import activity_type
from topo.base.cf import CFPLearningFn,CFPLF_Plugin
from topo.learningfn.projfn import CFPLF_PluginScaled
from topo.base.functionfamily import Hebbian,LearningFn
from topo.misc.inlinec import inline,provide_unoptimized_equivalent,\
     c_header,c_decorators
from topo.learningfn import BCMFixed

from projfn import CFPLF_Trace  # pyflakes:ignore (optimized version provided)




class CFPLF_Hebbian_opt(CFPLearningFn):
    """
    CF-aware Hebbian learning rule.

    Implemented in C for speed.  Should be equivalent to
    CFPLF_Plugin(single_cf_fn=Hebbian), except faster.

    As a side effect, sets the norm_total attribute on any cf whose
    weights are updated during learning, to speed up later operations
    that might depend on it.

    May return without modifying anything if the learning rate turns
    out to be zero.
    """
    single_cf_fn = param.ClassSelector(LearningFn,default=Hebbian(),readonly=True)
    
    def __call__(self, iterator, input_activity, output_activity, learning_rate, **params):
        single_connection_learning_rate = self.constant_sum_connection_rate(iterator.proj_n_units,learning_rate)
        if single_connection_learning_rate==0:
            return

        cfs = iterator.flatcfs
        num_cfs = len(cfs)  # pyflakes:ignore (passed to weave C code)
        irows,icols = input_activity.shape
        cf_type = iterator.cf_type  # pyflakes:ignore (passed to weave C code)

        # CEBALERT: this function *always* skips inactive units,
        # because it uses the output_activity directly rather than
        # going through the iterator. That's ok since we know this
        # function can always skip inactive units. But the unoptimized
        # equivalent should be made to do the same, because right now
        # it respects the iterator.  (Just a case of setting the
        # iterator's active_units_mask to be True before calling the
        # iterator in the unoptimized version.)

        sheet_mask = iterator.get_sheet_mask()  # pyflakes:ignore (passed to weave C code)

        code = c_header + """
            DECLARE_SLOT_OFFSET(weights,cf_type);
            DECLARE_SLOT_OFFSET(input_sheet_slice,cf_type);
            DECLARE_SLOT_OFFSET(mask,cf_type);
            DECLARE_SLOT_OFFSET(_norm_total,cf_type);
            DECLARE_SLOT_OFFSET(_has_norm_total,cf_type);

            %(cfs_loop_pragma)s
            for (int r=0; r<num_cfs; ++r) {
                double load = output_activity[r];
                if (load != 0 && sheet_mask[r] != 0) {
                    load *= single_connection_learning_rate;

                    PyObject *cf = PyList_GetItem(cfs,r);

                    LOOKUP_FROM_SLOT_OFFSET(float,weights,cf);
                    LOOKUP_FROM_SLOT_OFFSET(int,input_sheet_slice,cf);
                    LOOKUP_FROM_SLOT_OFFSET(float,mask,cf);

                    UNPACK_FOUR_TUPLE(int,rr1,rr2,cc1,cc2,input_sheet_slice);

                    double total = 0.0;

                    // modify non-masked weights
                    npfloat *inpj = input_activity+icols*rr1+cc1;
                    for (int i=rr1; i<rr2; ++i) {
                        npfloat *inpi = inpj;
                        for (int j=cc1; j<cc2; ++j) {
                            // The mask is floating point, so we have to 
                            // use a robust comparison instead of testing 
                            // against exactly 0.0.
                            if (*(mask++) >= 0.000001) {
                                *weights += load * *inpi;
                                total += fabs(*weights);
                            }
                            ++weights;
                            ++inpi;
                        }
                        inpj += icols;
                    }                    
                    // store the sum of the cf's weights
                    LOOKUP_FROM_SLOT_OFFSET(double,_norm_total,cf);
                    _norm_total[0]=total;
                    LOOKUP_FROM_SLOT_OFFSET(int,_has_norm_total,cf);
                    _has_norm_total[0]=1;
                }
            }
        """%c_decorators

        inline(code, ['input_activity', 'output_activity','sheet_mask','num_cfs',
                      'icols', 'cfs', 'single_connection_learning_rate','cf_type'],
               local_dict=locals(),
               headers=['<structmember.h>'])               


class CFPLF_Hebbian(CFPLF_Plugin):
    """Same as CFPLF_Plugin(single_cf_fn=Hebbian()); just for non-optimized fallback."""
    single_cf_fn = param.ClassSelector(LearningFn,default=Hebbian(),readonly=True)
provide_unoptimized_equivalent("CFPLF_Hebbian_opt","CFPLF_Hebbian",locals())


# CBERRORALERT: classes from here on probably ignore the sheet mask 

# JABALERT: Is this really a fixed-threshold BCM rule?  If so, is that really useful?
class CFPLF_BCMFixed_opt(CFPLearningFn):
    """
    CF-aware BCM learning rule.

    Implemented in C for speed.  Should be equivalent to
    BCMFixed for CF sheets, except faster.  

    As a side effect, sets the norm_total attribute on any cf whose
    weights are updated during learning, to speed up later operations
    that might depend on it.

    May return without modifying anything if the learning rate turns
    out to be zero.
    """
    
    unit_threshold=param.Number(default=0.5,bounds=(0,None),doc="Threshold between LTD and LTP.")
    
    def __call__(self, iterator, input_activity, output_activity, learning_rate, **params):
        rows,cols = output_activity.shape
        cfs = iterator.flatcfs
        num_cfs = len(cfs)  # pyflakes:ignore (passed to weave C code)
        single_connection_learning_rate = self.constant_sum_connection_rate(iterator.proj_n_units,learning_rate)
        if single_connection_learning_rate==0:
            return
        
        unit_threshold=self.unit_threshold  # pyflakes:ignore (passed to weave C code)
        
        irows,icols = input_activity.shape
        cf_type = iterator.cf_type  # pyflakes:ignore (passed to weave C code)
        code = c_header + """
            // CEBALERT: should provide a macro for getting offset

            ///// GET WEIGHTS OFFSET
            PyMemberDescrObject *weights_descr = (PyMemberDescrObject *)PyObject_GetAttrString(cf_type,"weights");
            Py_ssize_t weights_offset = weights_descr->d_member->offset;
            Py_DECREF(weights_descr);

            ///// GET SLICE OFFSET
            PyMemberDescrObject *slice_descr = (PyMemberDescrObject *)PyObject_GetAttrString(cf_type,"input_sheet_slice");
            Py_ssize_t slice_offset = slice_descr->d_member->offset;
            Py_DECREF(slice_descr);

            ///// GET MASK OFFSET
            PyMemberDescrObject *mask_descr = (PyMemberDescrObject *)PyObject_GetAttrString(cf_type,"mask");
            Py_ssize_t mask_offset = mask_descr->d_member->offset;
            Py_DECREF(mask_descr);


            npfloat *x = output_activity;

            for (int r=0; r<num_cfs; ++r) {
                double load = *x++;
                double unit_activity= load;
                if (load != 0) {
                    load *= single_connection_learning_rate;

                    PyObject *cf = PyList_GetItem(cfs,r);

                    PyArrayObject *weights_obj = *((PyArrayObject **)((char *)cf + weights_offset));
                    PyArrayObject *slice_obj = *((PyArrayObject **)((char *)cf + slice_offset));
                    PyArrayObject *mask_obj = *((PyArrayObject **)((char *)cf + mask_offset));

                    float *wi = (float *)(weights_obj->data);
                    int *slice = (int *)(slice_obj->data);
                    float *m = (float *)(mask_obj->data);

                    int rr1 = *slice++;
                    int rr2 = *slice++;
                    int cc1 = *slice++;
                    int cc2 = *slice;

                    double total = 0.0;

                    // modify non-masked weights
                    npfloat *inpj = input_activity+icols*rr1+cc1;
                    for (int i=rr1; i<rr2; ++i) {
                        npfloat *inpi = inpj;
                        for (int j=cc1; j<cc2; ++j) {
                            // The mask is floating point, so we have to 
                            // use a robust comparison instead of testing 
                            // against exactly 0.0.
                            if (*(m++) >= 0.000001) {
                                *wi += load * *inpi * (unit_activity - unit_threshold);
                                if (*wi<0) { *wi = 0;}
                                total += fabs(*wi);
                            }
                            ++wi;
                            ++inpi;
                        }
                        inpj += icols;
                    }

                    // store the sum of the cf's weights
                    PyObject *total_obj = PyFloat_FromDouble(total);  //(new ref)
                    PyObject_SetAttrString(cf,"_norm_total",total_obj);
                    PyObject_SetAttrString(cf,"_has_norm_total",Py_True);
                    Py_DECREF(total_obj);
                }
            }
        """

        inline(code, ['input_activity', 'output_activity','num_cfs',
                      'icols', 'cfs', 'single_connection_learning_rate',
                      'unit_threshold','cf_type'],
               local_dict=locals(),
               headers=['<structmember.h>'])               


class CFPLF_BCMFixed(CFPLF_Plugin):
    """Same as CFPLF_Plugin(single_cf_fn=BCMFixed()); just for non-optimized fallback."""
    single_cf_fn = param.ClassSelector(LearningFn,default=BCMFixed(),readonly=True)
provide_unoptimized_equivalent("CFPLF_BCMFixed_opt","CFPLF_Hebbian",locals())


# CEBALERT: 2009/04/03 - when used in GCA-LISSOM, causes Python to crash.
class CFPLF_Scaled_opt(CFPLF_PluginScaled):
    """
    CF-aware Scaled Hebbian learning rule.

    Implemented in C for speed.  Should be equivalent to
    CFPLF_PluginScaled(single_cf_fn=Hebbian), except faster.  

    As a side effect, sets the norm_total attribute on any cf whose
    weights are updated during learning, to speed up later operations
    that might depend on it.
    """
    single_cf_fn = param.ClassSelector(LearningFn,default=Hebbian(),readonly=True)
    
    def __call__(self, iterator, input_activity, output_activity, learning_rate, **params):
        
        if self.learning_rate_scaling_factor is None:
            self.learning_rate_scaling_factor = ones(output_activity.shape)*1.0

        learning_rate_scaling_factor = self.learning_rate_scaling_factor  # pyflakes:ignore (passed to weave C code)

        cfs = iterator.flatcfs
        num_cfs = len(cfs)  # pyflakes:ignore (passed to weave C code)
        single_connection_learning_rate = self.constant_sum_connection_rate(iterator.proj_n_units,learning_rate)
        if single_connection_learning_rate==0:
            return
        
        irows,icols = input_activity.shape
        code = c_header + """
            npfloat *x = output_activity;
            double *sclr = learning_rate_scaling_factor;

            for (int r=0; r<num_cfs; ++r) {
                double load = *x++;
                double  a= *sclr++;
                load = load * a;
                if (load != 0) {
                    load *= single_connection_learning_rate;
                    PyObject *cf = PyList_GetItem(cfs,r);
                    PyObject *weights_obj = PyObject_GetAttrString(cf,"weights");
                    PyObject *slice_obj   = PyObject_GetAttrString(cf,"input_sheet_slice");
                    PyObject *mask_obj    = PyObject_GetAttrString(cf,"mask");

                    float *wi = (float *)(((PyArrayObject*)weights_obj)->data);
                    int *slice =  (int *)(((PyArrayObject*)slice_obj)->data);
                    float *m  = (float *)(((PyArrayObject*)mask_obj)->data);

                    int rr1 = *slice++;
                    int rr2 = *slice++;
                    int cc1 = *slice++;
                    int cc2 = *slice;

                    double total = 0.0;

                    // modify non-masked weights
                    npfloat *inpj = input_activity+icols*rr1+cc1;
                    for (int i=rr1; i<rr2; ++i) {
                        npfloat *inpi = inpj;
                        for (int j=cc1; j<cc2; ++j) {
                            // The mask is floating point, so we have to 
                            // use a robust comparison instead of testing 
                            // against exactly 0.0.
                            if (*(m++) >= 0.000001) {
                                *wi += load * *inpi;
                                total += fabs(*wi);
                            }
                            ++wi;
                            ++inpi;
                        }
                        inpj += icols;
                    }

                    // Anything obtained with PyObject_GetAttrString must be explicitly freed
                    Py_DECREF(weights_obj);
                    Py_DECREF(slice_obj);
                    Py_DECREF(mask_obj);

                    // store the sum of the cf's weights
                    PyObject *total_obj = PyFloat_FromDouble(total);  //(new ref)
                    PyObject_SetAttrString(cf,"_norm_total",total_obj);
                    PyObject_SetAttrString(cf,"_has_norm_total",Py_True);
                    Py_DECREF(total_obj);
                }
            }
            
        """

        inline(code, ['input_activity','learning_rate_scaling_factor', 'output_activity','num_cfs', 'icols', 'cfs', 'single_connection_learning_rate'], local_dict=locals())


class CFPLF_Scaled(CFPLF_PluginScaled):
    """Same as CFPLF_PluginScaled(single_cf_fn=Hebbian()); just for non-optimized fallback."""
    single_cf_fn = param.ClassSelector(LearningFn,default=Hebbian(),readonly=True)
provide_unoptimized_equivalent("CFPLF_Scaled_opt","CFPLF_Scaled",locals())



class CFPLF_Trace_opt(CFPLearningFn):
    """
    Optimized version of CFPLF_Trace; see projfn.py for more info 
    """

    trace_strength=param.Number(default=0.5,bounds=(0.0,1.0),doc="""
       How much the learning is dominated by the activity trace, relative to the current value.""")

    single_cf_fn = param.ClassSelector(LearningFn,default=Hebbian(),readonly=True,
        doc="LearningFn that will be applied to each CF individually.")              

    def __call__(self, iterator, input_activity, output_activity, learning_rate, **params):
        cfs = iterator.flatcfs  # pyflakes:ignore (passed to weave C code)
        single_connection_learning_rate = self.constant_sum_connection_rate(iterator.proj_n_units,learning_rate)
        irows,icols = input_activity.shape
        
        if single_connection_learning_rate==0:
            return
        
        ##Initialise traces to zero if they don't already exist
        if not hasattr(self,'traces'):
            self.traces=zeros(output_activity.shape,activity_type)
        
        self.traces = (self.trace_strength*output_activity)+((1-self.trace_strength)*self.traces)
        traces = self.traces  # pyflakes:ignore (passed to weave C code)
        
        code = c_header + """
            npfloat *x = traces;

            for (int r=0; r<num_cfs; ++r) {
                double load = *x++;
                if (load != 0) {
                    load *= single_connection_learning_rate;
                    PyObject *cf = PyList_GetItem(cfs,r);
                    PyObject *weights_obj = PyObject_GetAttrString(cf,"weights");
                    PyObject *slice_obj   = PyObject_GetAttrString(cf,"input_sheet_slice");
                    PyObject *mask_obj    = PyObject_GetAttrString(cf,"mask");

                    float *wi = (float *)(((PyArrayObject*)weights_obj)->data);
                    int *slice =  (int *)(((PyArrayObject*)slice_obj)->data);
                    float *m  = (float *)(((PyArrayObject*)mask_obj)->data);

                    int rr1 = *slice++;
                    int rr2 = *slice++;
                    int cc1 = *slice++;
                    int cc2 = *slice;

                    double total = 0.0;

                    // modify non-masked weights
                    npfloat *inpj = input_activity+icols*rr1+cc1;
                    for (int i=rr1; i<rr2; ++i) {
                        npfloat *inpi = inpj;
                        for (int j=cc1; j<cc2; ++j) {
                            // The mask is floating point, so we have to 
                            // use a robust comparison instead of testing 
                            // against exactly 0.0.
                            if (*(m++) >= 0.000001) {
                                *wi += load * *inpi;
                                total += fabs(*wi);
                            }
                            ++wi;
                            ++inpi;
                        }
                        inpj += icols;
                    }

                    // Anything obtained with PyObject_GetAttrString must be explicitly freed
                    Py_DECREF(weights_obj);
                    Py_DECREF(slice_obj);
                    Py_DECREF(mask_obj);

                    // store the sum of the cf's weights
                    PyObject *total_obj = PyFloat_FromDouble(total);  //(new ref)
                    PyObject_SetAttrString(cf,"norm_total",total_obj);
                    PyObject_SetAttrString(cf,"_has_norm_total",Py_True);
                    Py_DECREF(total_obj);
                }
            }
        """

        inline(code, ['input_activity', 'traces','num_cfs', 'icols', 'cfs', 'single_connection_learning_rate'], local_dict=locals())


provide_unoptimized_equivalent("CFPLF_Trace_opt","CFPLF_Trace",locals())



"""
Response functions and CFProjection response functions (see projfn.py) written
in C to optimize performance.

Requires the weave package; without it unoptimized versions are used.

$Id$
"""

import param

from topo.base.functionfamily import ResponseFn,DotProduct
from topo.base.cf import CFPResponseFn, CFPRF_Plugin
from topo.misc.inlinec import inline,provide_unoptimized_equivalent,\
     c_header,c_decorators
from topo.misc.pyxhandler import provide_unoptimized_equivalent_cy
from topo.responsefn.projfn import CFPRF_EuclideanDistance  # pyflakes:ignore (optimized version provided)


# CEBALERT: this function works for 1D arrays; the docstring below is
# out of date. Need to update for this and other optimized fns that
# have been flattened.

class CFPRF_DotProduct_opt(CFPResponseFn):
    """
    Dot-product response function.

    Written in C for a manyfold speedup; see CFPRF_DotProduct for an
    easier-to-read version in Python.  The unoptimized Python version
    is equivalent to this one, but it also works for 1D arrays.
    """

    single_cf_fn = param.ClassSelector(ResponseFn,DotProduct(),readonly=True)    

    def __call__(self, iterator, input_activity, activity, strength, **params):
       
        temp_act = activity  # pyflakes:ignore (passed to weave C code)
        irows,icols = input_activity.shape
        X = input_activity.ravel()  # pyflakes:ignore (passed to weave C code)
        cfs = iterator.flatcfs
        num_cfs = len(cfs)  # pyflakes:ignore (passed to weave C code)
        mask = iterator.mask.data  # pyflakes:ignore (passed to weave C code)

        cf_type = iterator.cf_type  # pyflakes:ignore (passed to weave C code)

        # Note: no performance hit from array indexing of mask and
        # temp_act (r11447).
        code = c_header + """
            DECLARE_SLOT_OFFSET(weights,cf_type);
            DECLARE_SLOT_OFFSET(input_sheet_slice,cf_type);

            %(cfs_loop_pragma)s
            for (int r=0; r<num_cfs; ++r) {
                if(mask[r] == 0.0) {
                    temp_act[r] = 0;
                } else {
                    PyObject *cf = PyList_GetItem(cfs,r);

                    // CONTIGUOUS_ARRAY_FROM_SLOT_OFFSET(float,weights,cf) <<<<<<<<<<<

                    LOOKUP_FROM_SLOT_OFFSET_UNDECL_DATA(float,weights,cf);
                    char *data = weights_obj->data;                  
                    int s0 = weights_obj->strides[0];
                    int s1 = weights_obj->strides[1];

                    LOOKUP_FROM_SLOT_OFFSET(int,input_sheet_slice,cf);

                    UNPACK_FOUR_TUPLE(int,rr1,rr2,cc1,cc2,input_sheet_slice);

                    double tot = 0.0;
                    npfloat *xj = X+icols*rr1+cc1;

                    // computes the dot product
                    for (int i=rr1; i<rr2; ++i) {
                        npfloat *xi = xj;


                    //    float *wi = weights;                       
                    //    for (int j=cc1; j<cc2; ++j) {
                    //        tot += *wi * *xi;
                    //        ++wi;
                    //        ++xi;
                    //    }


                   for (int j=cc1; j<cc2; ++j) {
                      tot += *((float *)(data + (i-rr1)*s0 + (j-cc1)*s1)) * *xi;
                      ++xi;
                   }

                        xj += icols;
                 //       weights += cc2-cc1;
                    }  
                    temp_act[r] = tot*strength;

                //    DECREF_CONTIGUOUS_ARRAY(weights);
                }
            }
        """%c_decorators
        inline(code, ['mask','X', 'strength', 'icols', 'temp_act','cfs','num_cfs','cf_type'], 
               local_dict=locals(), headers=['<structmember.h>'])

class CFPRF_DotProduct(CFPRF_Plugin):
    """
    Wrapper written to allow transparent non-optimized fallback; 
    equivalent to CFPRF_Plugin(single_cf_fn=DotProduct()).
    """
    # CB: should probably have single_cf_fn here & readonly
    def __init__(self,**params):
        super(CFPRF_DotProduct,self).__init__(single_cf_fn=DotProduct(),**params)

provide_unoptimized_equivalent("CFPRF_DotProduct_opt","CFPRF_DotProduct",locals())


try:
    from optimized_cy import CFPRF_DotProduct_cyopt  # pyflakes:ignore (optimized version)
except:
    pass

provide_unoptimized_equivalent_cy("CFPRF_DotProduct_cyopt","CFPRF_DotProduct",locals())



# CEBERRORALERT: ignores the sheet mask!
class CFPRF_EuclideanDistance_opt(CFPResponseFn):
    """
    Euclidean-distance response function.

    Written in C for a several-hundred-times speedup; see
    CFPRF_EuclideanDistance for an easier-to-read (but otherwise
    equivalent) version in Python.
    """
    def __call__(self, iterator, input_activity, activity, strength, **params):
        temp_act = activity  # pyflakes:ignore (passed to weave C code)
        rows,cols = activity.shape
        irows,icols = input_activity.shape
        X = input_activity.ravel()  # pyflakes:ignore (passed to weave C code)
        cfs = iterator.flatcfs
        num_cfs = len(cfs)  # pyflakes:ignore (passed to weave C code)

        code = c_header + """
            #include <math.h>
            npfloat *tact = temp_act;
            double max_dist=0.0;
    
            for (int r=0; r<num_cfs; ++r) {
                PyObject *cf = PyList_GetItem(cfs,r);

                PyObject *weights_obj = PyObject_GetAttrString(cf,"weights");
                PyObject *slice_obj   = PyObject_GetAttrString(cf,"input_sheet_slice");

                float *wj = (float *)(((PyArrayObject*)weights_obj)->data);
                int *slice =  (int *)(((PyArrayObject*)slice_obj)->data);

                int rr1 = *slice++;
                int rr2 = *slice++;
                int cc1 = *slice++;
                int cc2 = *slice;

                npfloat *xj = X+icols*rr1+cc1;

                // computes the dot product
                double tot = 0.0;
                for (int i=rr1; i<rr2; ++i) {
                    npfloat *xi = xj;                        
                    float *wi = wj;
                    for (int j=cc1; j<cc2; ++j) {
                        double diff = *wi - *xi;
                        tot += diff*diff;
                        ++wi;
                        ++xi;
                    }
                    xj += icols;
                    wj += cc2-cc1;
                }

                double euclidean_distance = sqrt(tot); 
                if (euclidean_distance>max_dist)
                    max_dist = euclidean_distance;

                *tact = euclidean_distance;
                ++tact;

                // Anything obtained with PyObject_GetAttrString must be explicitly freed
                Py_DECREF(weights_obj);
                Py_DECREF(slice_obj);
            }
            tact = temp_act;
            for (int r=0; r<num_cfs; ++r) {
                *tact = strength*(max_dist - *tact);
                ++tact;
            }   
        """
        inline(code, ['X', 'strength', 'icols', 'temp_act','cfs','num_cfs'],
               local_dict=locals())

provide_unoptimized_equivalent("CFPRF_EuclideanDistance_opt","CFPRF_EuclideanDistance",locals())



"""
Projection classes.

A Projection is a connection between two Sheets, generally implemented
as a large set of ConnectionFields.

Any new Projection classes added to this directory will automatically
become available for any model.

$Id$
"""

from copy import copy

from numpy import exp,ones,zeros,array,nonzero

import param

# So all Projections are present in this package
from topo.base.projection import Projection
from topo.base.boundingregion import BoundingBox
from topo.base.sheet import activity_type
from topo.base.sheetcoords import Slice
from topo.base.cf import CFProjection,ConnectionField,MaskedCFIter,\
     CFPLearningFn,CFPLF_Identity,CFPOutputFn,CFIter,ResizableCFProjection
from topo.base.patterngenerator import PatternGenerator,Constant
from topo.base.functionfamily import CoordinateMapperFn,IdentityMF
from topo.misc.util import rowcol2idx
from topo.transferfn import TransferFn,IdentityTF
from topo.learningfn import LearningFn,IdentityLF
from topo.base import patterngenerator

class CFPOF_SharedWeight(CFPOutputFn):
    """
    CFPOutputFn for use with SharedWeightCFProjections.

    Applies the single_cf_fn to the single shared CF's weights.
    """
    single_cf_fn = param.ClassSelector(TransferFn,default=IdentityTF())
    
    # CEBALERT: remove norm_values?
    def __call__(self, cfs, norm_values=None, **params):
        """Apply the specified single_cf_fn to every CF."""
        if type(self.single_cf_fn) is not IdentityTF:
            cf = cfs[0,0]
            self.single_cf_fn(cf.weights)

from topo.base.cf import _create_mask

class SharedWeightCF(ConnectionField):
    
    __slots__ = []
        
    def __init__(self,cf,input_sheet,x=0.0,y=0.0,template=BoundingBox(radius=0.1),
                 mask=patterngenerator.Constant(),
                 min_matrix_radius=1):
        """
        From an existing copy of ConnectionField (CF) that acts as a
        template, create a new CF that shares weights with the
        template CF.  Copies all the properties of CF to stay
        identical except the weights variable that actually contains
        the data.
        
        The only difference from a normal CF is that the weights of
        the CF are implemented as a numpy view into the single master
        copy of the weights stored in the CF template.
        """
        # CEBALERT: There's no call to super's __init__; see JAHACKALERT
        # below.
        template = copy(template)

        if not isinstance(template,Slice):
            template = Slice(template,input_sheet,force_odd=True,
                             min_matrix_radius=min_matrix_radius)

        # Note: if passed in, mask is shared between CFs (but not if created here)
        if not hasattr(mask,'view'):
            mask = _create_mask(patterngenerator.Constant(),
                               template.compute_bounds(input_sheet),
                               input_sheet,True,0.5) 



        self._has_norm_total=False
        self.mask=mask 
        weights_slice = self._create_input_sheet_slice(input_sheet,x,y,template,min_matrix_radius=min_matrix_radius)
        self.weights = weights_slice.submatrix(cf.weights)
        
        # JAHACKALERT the TransferFn cannot be applied in SharedWeightCF
        # - another inconsistency in the class tree design - there
        # should be nothing in the parent class that is ignored in its
        # children.  Probably need to extract some functionality of
        # ConnectionField into a shared abstract parent class.
        # We have agreed to make this right by adding a constant property that
        # will be set true if the learning should be active
        # The SharedWeightCFProjection class and its anccestors will
        # have this property set to false which means that the 
        # learning will be deactivated

        

class SharedWeightCFProjection(CFProjection):
    """
    A Projection with a single set of weights, shared by all units.

    Otherwise similar to CFProjection, except that learning is
    currently disabled.
    """
    ### JABHACKALERT: Set to be constant as a clue that learning won't
    ### actually work yet, but we could certainly extend it to support
    ### learning if desired, e.g. to learn position-independent responses.
    learning_fn = param.ClassSelector(CFPLearningFn,CFPLF_Identity(),constant=True)
    weights_output_fns = param.HookList(default=[CFPOF_SharedWeight()])
    precedence = param.Number(default=0.5)

    def __init__(self,**params):
        """
        Initialize the Projection with a single cf_type object
        (typically a ConnectionField),
        """
        # We don't want the whole set of cfs initialized, but we
        # do want anything that CFProjection defines.
        super(SharedWeightCFProjection,self).__init__(initialize_cfs=False,**params)

        # We want the sharedcf to be located on the grid, so use the
        # center of a unit
        sheet_rows,sheet_cols=self.src.shape
        # arbitrary (e.g. could use 0,0) 
        center_row,center_col = sheet_rows/2,sheet_cols/2
        center_unitxcenter,center_unitycenter=self.src.matrixidx2sheet(center_row,
                                                                       center_col)

        
        self.__sharedcf=self.cf_type(self.src,
                                     x=center_unitxcenter,
                                     y=center_unitycenter,
                                     template=self._slice_template,
                                     weights_generator=self.weights_generator,
                                     mask=self.mask_template,
                                     output_fns=[wof.single_cf_fn for wof in self.weights_output_fns],
                                     min_matrix_radius=self.min_matrix_radius)

        self._create_cfs()



    def _create_cf(self,x,y):
        x_cf,y_cf = self.coord_mapper(x,y)
        # Does not pass the mask, as it would have to be sliced
        # for each cf, and is only used for learning.
        CF = SharedWeightCF(self.__sharedcf,self.src,x=x_cf,y=y_cf, #JUDE ADDED
                            template=self._slice_template,
                            min_matrix_radius=self.min_matrix_radius,
                            mask=self.mask_template)

        return CF
            
    
    def learn(self):
        """
        Because of how output functions are applied, it is not currently
        possible to use learning functions and learning output functions for
        SharedWeightCFProjections, so we disable them here.
        """
        pass
    
    
    def apply_learn_output_fns(self,active_units_mask=True):
        """
        Because of how output functions are applied, it is not currently
        possible to use learning functions and learning output functions for
        SharedWeightCFProjections, so we disable them here.
        """
        pass


    def n_bytes(self):
        return self.activity.nbytes + self.__sharedcf.weights.nbytes + \
               sum([cf.input_sheet_slice.nbytes
                    for cf,i in CFIter(self)()])
                   




# JABALERT: Can this be replaced with a CFProjection with a Hysteresis output_fn?
# If not it should probably be replaced with a new output_fn type instead.
class LeakyCFProjection(ResizableCFProjection):
    """
    A projection that has a decay_rate parameter so that incoming
    input is decayed over time as x(t) = input + x(t-1)*exp(-decay_rate),
    and then the weighted sum of x(t) is calculated.
    """

    decay_rate = param.Number(default=1.0,bounds=(0,None),
                        doc="Input decay rate for each leaky synapse")

    precedence = param.Number(default=0.4)
    
    def __init__(self,**params):
        super(LeakyCFProjection,self).__init__(**params)
        self.leaky_input_buffer = zeros(self.src.activity.shape)

    def activate(self,input_activity):
        """
        Retain input_activity from the previous step in leaky_input_buffer
        and add a leaked version of it to the current input_activity. This 
        function needs to deal with a finer time-scale.
        """
        self.leaky_input_buffer = input_activity + self.leaky_input_buffer*exp(-self.decay_rate) 
        super(LeakyCFProjection,self).activate(self.leaky_input_buffer)

    def n_bytes(self):
        return super(LeakyCFProjection,self).n_bytes() + \
               self.activity.nbytes*1 # for leaky_input_buffer


class ScaledCFProjection(CFProjection):
    """
    Allows scaling of activity based on a specified target average activity.
    
    An exponentially weighted average is used to calculate the average
    activity.  This average is then used to calculate scaling factors
    for the current activity and for the learning rate.

    The plastic parameter can be used to turn off updating of the average
    activity, e.g. during map measurement.
    """

    target = param.Number(default=0.045, doc="""Target average activity for the projection.""")

    target_lr = param.Number(default=0.045, doc="""
        Target average activity for scaling the learning rate.""")
    
    smoothing = param.Number(default=0.999, doc="""
        Influence of previous activity, relative to current, for computing the average.""")
    precedence = param.Number(default=0.4)

    def __init__(self,**params):
        # JABALERT: Private variables should be prefixed __ (truly private) or _ (nominally private)
        # Are all of these really something you expect people to be using outside this class?
        super(ScaledCFProjection,self).__init__(**params)
        self.x_avg=None
        self.sf=None
        self.lr_sf=None
        self.scaled_x_avg=None


    def calculate_sf(self):
        """
        Calculate current scaling factors based on the target and previous average activities.

        Keeps track of the scaled average for debugging. Could be
        overridden by a subclass to calculate the factors differently.
        """
      
        if self.plastic:
            self.sf *=0.0
            self.lr_sf *=0.0
            self.sf += self.target/self.x_avg
            self.lr_sf += self.target_lr/self.x_avg
            self.x_avg = (1.0-self.smoothing)*self.activity + self.smoothing*self.x_avg
            self.scaled_x_avg = (1.0-self.smoothing)*self.activity*self.sf + self.smoothing*self.scaled_x_avg


    def do_scaling(self):
        """
        Scale the projection activity and learning rate for the projection.
        """
        self.activity *= self.sf
        if hasattr(self.learning_fn,'learning_rate_scaling_factor'):
            self.learning_fn.update_scaling_factor(self.lr_sf)
        else:
            raise ValueError("Projections to be called must have learning function which supports scaling (e.g. CFPLF_PluginScaled).")

                   
    def activate(self,input_activity):
        """Activate using the specified response_fn and output_fn."""
        self.input_buffer = input_activity
        self.activity *=0.0

        if self.x_avg is None:
            self.x_avg=self.target*ones(self.dest.shape, activity_type)
        if self.scaled_x_avg is None:
            self.scaled_x_avg=self.target*ones(self.dest.shape, activity_type) 
        if self.sf is None:
            self.sf=ones(self.dest.shape, activity_type)
        if self.lr_sf is None:
            self.lr_sf=ones(self.dest.shape, activity_type)
        
        self.response_fn(MaskedCFIter(self), input_activity, self.activity, self.strength)
        for of in self.output_fns:
            of(self.activity)
        self.calculate_sf()
        self.do_scaling()


    def n_bytes(self):
        return super(ScaledCFProjection,self).n_bytes() + \
               self.activity.nbytes*4 # for x_avg,sf,lr_sf,scaled_x_avg



class OneToOneProjection(Projection):
    """
    A projection that has at most one input connection for each unit.

    This projection has exactly one weight for each destination unit.
    The input locations on the input sheet are determined by a
    coordinate mapper.  Inputs that map outside the bounds of the
    input sheet are treated as having zero weight.
    """
    coord_mapper = param.ClassSelector(CoordinateMapperFn,
        default=IdentityMF(),
        doc='Function to map a destination unit coordinate into the src sheet.')

    weights_generator = param.ClassSelector(PatternGenerator,
        default=Constant(),constant=True,
        doc="""Generate initial weight values for each unit of the destination sheet.""")

    learning_fn = param.ClassSelector(LearningFn,default=IdentityLF(),
        doc="""Learning function applied to weights.""")

    learning_rate = param.Number(default=0)

    
    def __init__(self,**kw):
        super(OneToOneProjection,self).__init__(**kw)

        self.input_buffer = None
        
        dx,dy = self.dest.bounds.centroid()

        # JPALERT: Not sure if this is the right way to generate weights.
        # For full specificity, each initial weight should be dependent on the
        # coordinates of both the src unit and the dest unit.
        self.weights = self.weights_generator(bounds=self.dest.bounds,
                                              xdensity=self.dest.xdensity,
                                              ydensity=self.dest.ydensity)


        # JPALERT: CoordMapperFns should really be made to take
        # matrices of x/y points and apply their mapping to all.  This
        # could give great speedups, esp for AffineTransform mappings,
        # which can be applied to many points with a single matrix
        # multiplication.
        srccoords = [self.coord_mapper(x,y) 
                     for y in reversed(self.dest.sheet_rows())
                     for x in self.dest.sheet_cols()]
        
        self.src_idxs = array([rowcol2idx(r,c,self.src.activity.shape)
                               for r,c in (self.src.sheet2matrixidx(u,v)
                                           for u,v in srccoords)])

        # dest_idxs contains the indices of the dest units whose weights project
        # in bounds on the src sheet.
        src_rows,src_cols = self.src.activity.shape
        def in_bounds(x,y):
            r,c = self.src.sheet2matrixidx(x,y)
            return (0 <= r < src_rows) and (0 <= c < src_cols)
        destmask = [in_bounds(x,y) for x,y in srccoords]

        # The [0] is required because numpy.nonzero returns the
        # nonzero indices wrapped in a one-tuple.
        self.dest_idxs = nonzero(destmask)[0]
        self.src_idxs = self.src_idxs.take(self.dest_idxs)
        assert len(self.dest_idxs) == len(self.src_idxs)

        self.activity = zeros(self.dest.shape,dtype=float)

    def activate(self,input):
        self.input_buffer = input
        result = self.weights.take(self.dest_idxs) * input.take(self.src_idxs) * self.strength
        self.activity.put(self.dest_idxs,result)
        for of in self.output_fns:
            of(self.activity)

    def learn(self):
        if self.input_buffer is not None:
            self.learning_fn(self.input_buffer,
                             self.dest.activity,
                             self.weights,
                             self.learning_rate)

    def n_conns(self):
        rows,cols=self.activity.shape
        return rows*cols
    
    def n_bytes(self):
        return super(OneToOneProjection,self).n_bytes() + \
               self.activity.nbytes

            
_public = list(set([_k for _k,_v in locals().items()
                    if isinstance(_v,type) and issubclass(_v,Projection)]))
_public += [
    "CFPOF_SharedWeight",
    "SharedWeightCF",
]

# Automatically discover all .py files in this directory.
import os,fnmatch
__all__ = _public + [f.split('.py')[0] for f in os.listdir(__path__[0]) if fnmatch.fnmatch(f,'[!._]*.py')]
del f,os,fnmatch



"""
FixedPoint objects support decimal arithmetic with a fixed number of
digits (called the object's precision) after the decimal point.  The
number of digits before the decimal point is variable & unbounded.

The precision is user-settable on a per-object basis when a FixedPoint
is constructed, and may vary across FixedPoint objects.  The precision
may also be changed after construction via FixedPoint.set_precision(p).
Note that if the precision of a FixedPoint is reduced via set_precision,
information may be lost to rounding.

>>> x = FixedPoint("5.55")  # precision defaults to 2
>>> print x
5.55
>>> x.set_precision(1)      # round to one fraction digit
>>> print x
5.6
>>> print FixedPoint("5.55", 1)  # same thing setting to 1 in constructor
5.6
>>> repr(x) #  returns constructor string that reproduces object exactly
"FixedPoint('5.6', 1)"
>>>

When FixedPoint objects of different precision are combined via + - * /,
the result is computed to the larger of the inputs' precisions, which also
becomes the precision of the resulting FixedPoint object.

>>> print FixedPoint("3.42") + FixedPoint("100.005", 3)
103.425
>>>

When a FixedPoint is combined with other numeric types (ints, floats,
strings representing a number) via + - * /, then similarly the computation
is carried out using-- and the result inherits --the FixedPoint's
precision.

>>> print FixedPoint(1) / 7
0.14
>>> print FixedPoint(1, 30) / 7
0.142857142857142857142857142857
>>>

The string produced by str(x) (implictly invoked by "print") always
contains at least one digit before the decimal point, followed by a
decimal point, followed by exactly x.get_precision() digits.  If x is
negative, str(x)[0] == "-".

The FixedPoint constructor can be passed an int, long, string, float,
FixedPoint, or any object convertible to a float via float() or to a
long via long().  Passing a precision is optional; if specified, the
precision must be a non-negative int.  There is no inherent limit on
the size of the precision, but if very very large you'll probably run
out of memory.

Note that conversion of floats to FixedPoint can be surprising, and
should be avoided whenever possible.  Conversion from string is exact
(up to final rounding to the requested precision), so is greatly
preferred.

>>> print FixedPoint(1.1e30)
1099999999999999993725589651456.00
>>> print FixedPoint("1.1e30")
1100000000000000000000000000000.00
>>>

The following Python operators and functions accept FixedPoints in the
expected ways:

    binary + - * / % divmod
        with auto-coercion of other types to FixedPoint.
        + - % divmod  of FixedPoints are always exact.
        * / of FixedPoints may lose information to rounding, in
            which case the result is the infinitely precise answer
            rounded to the result's precision.
        divmod(x, y) returns (q, r) where q is a long equal to
            floor(x/y) as if x/y were computed to infinite precision,
            and r is a FixedPoint equal to x - q * y; no information
            is lost.  Note that q has the sign of y, and abs(r) < abs(y).
    unary -
    == != < > <= >=  cmp
    min  max
    float  int  long    (int and long truncate)
    abs
    str  repr
    hash
    use as dict keys
    use as boolean (e.g. "if some_FixedPoint:" -- true iff not zero)

Methods unique to FixedPoints:
   .copy()              return new FixedPoint with same value
   .frac()              long(x) + x.frac() == x
   .get_precision()     return the precision(p) of this FixedPoint object
   .set_precision(p)    set the precision of this FixedPoint object
   
Provided as-is; use at your own risk; no warranty; no promises; enjoy!
"""

# Released to the public domain 28-Mar-2001,
# by Tim Peters (tim.one@home.com).


# 28-Mar-01 ver 0.0,4
#     Use repr() instead of str() inside __str__, because str(long) changed
#     since this was first written (used to produce trailing "L", doesn't
#     now).
#
# 09-May-99 ver 0,0,3
#     Repaired __sub__(FixedPoint, string); was blowing up.
#     Much more careful conversion of float (now best possible).
#     Implemented exact % and divmod.
#
# 14-Oct-98 ver 0,0,2
#     Added int, long, frac.  Beefed up docs.  Removed DECIMAL_POINT
#     and MINUS_SIGN globals to discourage bloating this class instead
#     of writing formatting wrapper classes (or subclasses)
#
# 11-Oct-98 ver 0,0,1
#     posted to c.l.py

__copyright__ = "Copyright (C) Python Software Foundation"
__author__ = "Tim Peters"
__version__ = 0, 1, 0

def bankersRounding(self, dividend, divisor, quotient, remainder):
    """
    rounding via nearest-even
    increment the quotient if
         the remainder is more than half of the divisor
      or the remainder is exactly half the divisor and the quotient is odd
    """
    c = cmp(remainder << 1, divisor)
    # c < 0 <-> remainder < divisor/2, etc
    if c > 0 or (c == 0 and (quotient & 1) == 1):
        quotient += 1
    return quotient

def addHalfAndChop(self, dividend, divisor, quotient, remainder):
    """
    the equivalent of 'add half and chop'
    increment the quotient if
         the remainder is greater than half of the divisor
      or the remainder is exactly half the divisor and the quotient is >= 0
    """
    c = cmp(remainder << 1, divisor)
    # c < 0 <-> remainder < divisor/2, etc
    if c > 0 or (c == 0 and quotient >= 0):
        quotient += 1
    return quotient

# 2002-10-20 dougfort - fake classes for pre 2.2 compatibility
try:
    object
except NameError:
    class object:
        pass
    def property(x, y):
        return None

# The default value for the number of decimal digits carried after the
# decimal point.  This only has effect at instance initialization.
DEFAULT_PRECISION = 2

class FixedPoint(object):
    """Basic FixedPoint object class,
        The exact value is self.n / 10**self.p;
        self.n is a long; self.p is an int
    """
    __slots__ = ['n', 'p']

    def __init__(self, value=0, precision=None):
        self.n = self.p = 0
        if precision == None:
            precision = DEFAULT_PRECISION
           
        self.set_precision(precision)
        p = self.p

        if isinstance(value, type("42.3e5")):
            n, exp = _string2exact(value)
            # exact value is n*10**exp = n*10**(exp+p)/10**p
            effective_exp = exp + p
            if effective_exp > 0:
                n = n * _tento(effective_exp)
            elif effective_exp < 0:
                n = self._roundquotient(n, _tento(-effective_exp))
            self.n = n
            return

        if isinstance(value, type(42)) or isinstance(value, type(42L)):
            self.n = long(value) * _tento(p)
            return

        if isinstance(value, type(self)):
            temp = value.copy()
            temp.set_precision(p)
            self.n, self.p = temp.n, temp.p
            return

        if isinstance(value, type(42.0)):
            # XXX ignoring infinities and NaNs and overflows for now
            import math
            f, e = math.frexp(abs(value))
            assert f == 0 or 0.5 <= f < 1.0
            # |value| = f * 2**e exactly

            # Suck up CHUNK bits at a time; 28 is enough so that we suck
            # up all bits in 2 iterations for all known binary double-
            # precision formats, and small enough to fit in an int.
            CHUNK = 28
            top = 0L
            # invariant: |value| = (top + f) * 2**e exactly
            while f:
                f = math.ldexp(f, CHUNK)
                digit = int(f)
                assert digit >> CHUNK == 0
                top = (top << CHUNK) | digit
                f = f - digit
                assert 0.0 <= f < 1.0
                e = e - CHUNK

            # now |value| = top * 2**e exactly
            # want n such that n / 10**p = top * 2**e, or
            # n = top * 10**p * 2**e
            top = top * _tento(p)
            if e >= 0:
                n = top << e
            else:
                n = self._roundquotient(top, 1L << -e)
            if value < 0:
                n = -n
            self.n = n
            return

        if isinstance(value, type(42-42j)):
            raise TypeError("can't convert complex to FixedPoint: " +
                            `value`)

        # can we coerce to a float?
        yes = 1
        try:
            asfloat = float(value)
        except:
            yes = 0
        if yes:
            self.__init__(asfloat, p)
            return

        # similarly for long
        yes = 1
        try:
            aslong = long(value)
        except:
            yes = 0
        if yes:
            self.__init__(aslong, p)
            return

        raise TypeError("can't convert to FixedPoint: " + `value`)

    def get_precision(self):
        """Return the precision of this FixedPoint.

           The precision is the number of decimal digits carried after
           the decimal point, and is an int >= 0.
        """

        return self.p

    def set_precision(self, precision=DEFAULT_PRECISION):
        """Change the precision carried by this FixedPoint to p.

           precision must be an int >= 0, and defaults to
           DEFAULT_PRECISION.

           If precision is less than this FixedPoint's current precision,
           information may be lost to rounding.
        """

        try:
            p = int(precision)
        except:
            raise TypeError("precision not convertable to int: " +
                            `precision`)
        if p < 0:
            raise ValueError("precision must be >= 0: " + `precision`)

        if p > self.p:
            self.n = self.n * _tento(p - self.p)
        elif p < self.p:
            self.n = self._roundquotient(self.n, _tento(self.p - p))
        self.p = p

    precision = property(get_precision, set_precision)

    def __str__(self):
        n, p = self.n, self.p
        i, f = divmod(abs(n), _tento(p))
        if p:
            frac = repr(f)[:-1]
            frac = "0" * (p - len(frac)) + frac
        else:
            frac = ""
        return "-"[:n<0] + \
               repr(i)[:-1] + \
               "." + frac

    def __repr__(self):
        return "FixedPoint" + `(str(self), self.p)`

    def copy(self):
        return _mkFP(self.n, self.p, type(self))

    __copy__ = copy

    def __deepcopy__(self, memo):
        return self.copy()

    # Basic support for pickling
    def __getstate__(self):
        state={}
        try: 
            for k in self.__slots__:
                state[k] = getattr(self,k)
        except AttributeError:
            pass
        return state

    def __setstate__(self,state):
        for k,v in state.items():
            setattr(self,k,v)
        self.unpickle()

    def unpickle(self):
        pass


    def __cmp__(self, other):
        xn, yn, p = _norm(self, other, FixedPoint=type(self))
        return cmp(xn, yn)

    def __hash__(self):
        """ Caution!  == values must have equal hashes, and a FixedPoint
            is essentially a rational in unnormalized form.  There's
            really no choice here but to normalize it, so hash is
            potentially expensive.
            n, p = self.__reduce()

            Obscurity: if the value is an exact integer, p will be 0 now,
            so the hash expression reduces to hash(n).  So FixedPoints
            that happen to be exact integers hash to the same things as
            their int or long equivalents.  This is Good.  But if a
            FixedPoint happens to have a value exactly representable as
            a float, their hashes may differ.  This is a teensy bit Bad.
        """
        n, p = self.__reduce()
        return hash(n) ^ hash(p)

    def __nonzero__(self):
        """ Returns true if this FixedPoint is not equal to zero"""
        return self.n != 0

    def __neg__(self):
        return _mkFP(-self.n, self.p, type(self))

    def __abs__(self):
        """ Returns new FixedPoint containing the absolute value of this FixedPoint"""
        if self.n >= 0:
            return self.copy()
        else:
            return -self

    def __add__(self, other):
        n1, n2, p = _norm(self, other, FixedPoint=type(self))
        # n1/10**p + n2/10**p = (n1+n2)/10**p
        return _mkFP(n1 + n2, p, type(self))

    __radd__ = __add__

    def __sub__(self, other):
        if not isinstance(other, type(self)):
            other = type(self)(other, self.p)
        return self.__add__(-other)

    def __rsub__(self, other):
        return (-self) + other

    def __mul__(self, other):
        n1, n2, p = _norm(self, other, FixedPoint=type(self))
        # n1/10**p * n2/10**p = (n1*n2/10**p)/10**p
        return _mkFP(self._roundquotient(n1 * n2, _tento(p)), p, type(self))

    __rmul__ = __mul__

    def __div__(self, other):
        n1, n2, p = _norm(self, other, FixedPoint=type(self))
        if n2 == 0:
            raise ZeroDivisionError("FixedPoint division")
        if n2 < 0:
            n1, n2 = -n1, -n2
        # n1/10**p / (n2/10**p) = n1/n2 = (n1*10**p/n2)/10**p
        return _mkFP(self._roundquotient(n1 * _tento(p), n2), p, type(self))

    def __rdiv__(self, other):
        n1, n2, p = _norm(self, other, FixedPoint=type(self))
        return _mkFP(n2, p, FixedPoint=type(self)) / self

    def __divmod__(self, other):
        n1, n2, p = _norm(self, other, FixedPoint=type(self))
        if n2 == 0:
            raise ZeroDivisionError("FixedPoint modulo")
        # floor((n1/10**p)/(n2*10**p)) = floor(n1/n2)
        q = n1 / n2
        # n1/10**p - q * n2/10**p = (n1 - q * n2)/10**p
        return q, _mkFP(n1 - q * n2, p, type(self))

    def __rdivmod__(self, other):
        n1, n2, p = _norm(self, other, FixedPoint=type(self))
        return divmod(_mkFP(n2, p), self)

    def __mod__(self, other):
        return self.__divmod__(other)[1]

    def __rmod__(self, other):
        n1, n2, p = _norm(self, other, FixedPoint=type(self))
        return _mkFP(n2, p, type(self)).__mod__(self)

    def __float__(self):
        """Return the floating point representation of this FixedPoint. 
            Caution! float can lose precision.
        """
        n, p = self.__reduce()
        return float(n) / float(_tento(p))

    def __long__(self):
        """EJG/DF - Should this round instead?
            Note e.g. long(-1.9) == -1L and long(1.9) == 1L in Python
            Note that __int__ inherits whatever __long__ does,
                 and .frac() is affected too
        """
        answer = abs(self.n) / _tento(self.p)
        if self.n < 0:
            answer = -answer
        return answer

    def __int__(self):
        """Return integer value of FixedPoint object."""
        return int(self.__long__())
    
    def frac(self):
        """Return fractional portion as a FixedPoint.

           x.frac() + long(x) == x
        """
        return self - long(self)

    def _roundquotient(self, x, y):
        """
        Divide x by y,
        return the result of rounding
        Developers may substitute their own 'round' for custom rounding
        y must be > 0
        """
        assert y > 0
        n, leftover = divmod(x, y)
        return self.round(x, y, n, leftover)

    def __reduce(self):
        """ Return n, p s.t. self == n/10**p and n % 10 != 0"""
        n, p = self.n, self.p
        if n == 0:
            p = 0
        while p and n % 10 == 0:
            p = p - 1
            n = n / 10
        return n, p

# 2002-10-04 dougfort - Default to Banker's Rounding for backward compatibility
FixedPoint.round = bankersRounding

# return 10L**n

def _tento(n, cache={}):
    """Cached computation of 10**n"""
    try:
        return cache[n]
    except KeyError:
        answer = cache[n] = 10L ** n
        return answer

def _norm(x, y, isinstance=isinstance, FixedPoint=FixedPoint,
                _tento=_tento):
    """Return xn, yn, p s.t.
           p = max(x.p, y.p)
           x = xn / 10**p
           y = yn / 10**p

        x must be FixedPoint to begin with; if y is not FixedPoint,
        it inherits its precision from x.

        Note that this method is called a lot, so default-arg tricks are helpful.
    """
    assert isinstance(x, FixedPoint)
    if not isinstance(y, FixedPoint):
        y = FixedPoint(y, x.p)
    xn, yn = x.n, y.n
    xp, yp = x.p, y.p
    if xp > yp:
        yn = yn * _tento(xp - yp)
        p = xp
    elif xp < yp:
        xn = xn * _tento(yp - xp)
        p = yp
    else:
        p = xp  # same as yp
    return xn, yn, p

def _mkFP(n, p, FixedPoint=FixedPoint):
    """Make FixedPoint objext - Return a new FixedPoint object with the selected precision."""
    f = FixedPoint()
    #print '_mkFP Debug: %s, value=%s' % (type(f),n)
    f.n = n
    f.p = p
    return f

# crud for parsing strings
import re

# There's an optional sign at the start, and an optional exponent
# at the end.  The exponent has an optional sign and at least one
# digit.  In between, must have either at least one digit followed
# by an optional fraction, or a decimal point followed by at least
# one digit.  Yuck.

_parser = re.compile(r"""
    \s*
    (?P<sign>[-+])?
    (
        (?P<int>\d+) (\. (?P<frac>\d*))?
    |
        \. (?P<onlyfrac>\d+)
    )
    ([eE](?P<exp>[-+]? \d+))?
    \s* $
""", re.VERBOSE).match

del re


def _string2exact(s):
    """Return n, p s.t. float string value == n * 10**p exactly."""
    m = _parser(s)
    if m is None:
        raise ValueError("can't parse as number: " + `s`)

    exp = m.group('exp')
    if exp is None:
        exp = 0
    else:
        exp = int(exp)

    intpart = m.group('int')
    if intpart is None:
        intpart = "0"
        fracpart = m.group('onlyfrac')
    else:
        fracpart = m.group('frac')
        if fracpart is None or fracpart == "":
            fracpart = "0"
    assert intpart
    assert fracpart

    i, f = long(intpart), long(fracpart)
    nfrac = len(fracpart)
    i = i * _tento(nfrac) + f
    exp = exp - nfrac

    if m.group('sign') == "-":
        i = -i

    return i, exp

def _test():
    """Unit testing framework"""
    fp = FixedPoint
    o = fp("0.1")
    assert str(o) == "0.10"
    t = fp("-20e-2", 5)
    assert str(t) == "-0.20000"
    assert t < o
    assert o > t
    assert min(o, t) == min(t, o) == t
    assert max(o, t) == max(t, o) == o
    assert o != t
    assert --t == t
    assert abs(t) > abs(o)
    assert abs(o) < abs(t)
    assert o == o and t == t
    assert t.copy() == t
    assert o == -t/2 == -.5 * t
    assert abs(t) == o + o
    assert abs(o) == o
    assert o/t == -0.5
    assert -(t/o) == (-t)/o == t/-o == 2
    assert 1 + o == o + 1 == fp(" +00.000011e+5  ")
    assert 1/o == 10
    assert o + t == t + o == -o
    assert 2.0 * t == t * 2 == "2" * t == o/o * 2L * t
    assert 1 - t == -(t - 1) == fp(6L)/5
    assert t*t == 4*o*o == o*4*o == o*o*4
    assert fp(2) - "1" == 1
    assert float(-1/t) == 5.0
    for p in range(20):
        assert 42 + fp("1e-20", p) - 42 == 0
    assert 1/(42 + fp("1e-20", 20) - 42) == fp("100.0E18")
    o = fp(".9995", 4)
    assert 1 - o == fp("5e-4", 10)
    o.set_precision(3)
    assert o == 1
    o = fp(".9985", 4)
    o.set_precision(3)
    assert o == fp(".998", 10)
    assert o == o.frac()
    o.set_precision(100)
    assert o == fp(".998", 10)
    o.set_precision(2)
    assert o == 1
    x = fp(1.99)
    assert long(x) == -long(-x) == 1L
    assert int(x) == -int(-x) == 1
    assert x == long(x) + x.frac()
    assert -x == long(-x) + (-x).frac()
    assert fp(7) % 4 == 7 % fp(4) == 3
    assert fp(-7) % 4 == -7 % fp(4) == 1
    assert fp(-7) % -4 == -7 % fp(-4) == -3
    assert fp(7.0) % "-4.0" == 7 % fp(-4) == -1
    assert fp("5.5") % fp("1.1") == fp("5.5e100") % fp("1.1e100") == 0
    assert divmod(fp("1e100"), 3) == (long(fp("1e100")/3), 1)

if __name__ == '__main__':
    _test()

